Package: RPostgreSQL
Version: 0.7-8
Date: 2025-03-28
Title: R Interface to the 'PostgreSQL' Database System
Maintainer: Tomoaki Nishiyama <tomoaki@sci.u-toyama.ac.jp>
Description: Database interface and 'PostgreSQL' driver for 'R'.
 This package provides a Database Interface 'DBI' compliant 
 driver for 'R' to access 'PostgreSQL' database systems.  
 In order to build and install this package from source, 'PostgreSQL' 
 itself must be present your system to provide 'PostgreSQL' functionality 
 via its libraries and header files. These files are provided as
 'postgresql-devel' package under some Linux distributions.
 On 'macOS' and 'Microsoft Windows' system the attached 'libpq' library source will be used.
LazyLoad: true
Depends: R (>= 3.4.0), methods, DBI (>= 0.3)
License: GPL-3 | file LICENSE
Copyright: Authors listed above, PostgreSQL Global Development Group,
        and The Regents of the University of California
Collate: S4R.R zzz.R PostgreSQLSupport.R dbObjectId.R PostgreSQL.R
URL: https://github.com/tomoakin/RPostgreSQL,
        https://cran.r-project.org/package=DBI,
        https://www.postgresql.org
Authors@R: c(person(given = "Joe",
                       family = "Conway",
                       role = "aut"),
                person(given = "Dirk",
                       family = "Eddelbuettel",
                       role = "aut"),
                person(given = "Tomoaki",
                       family = "Nishiyama",
                       role = c("aut", "cre"),
                       email = "tomoaki@sci.u-toyama.ac.jp"),
                person(given = c("Sameer", "Kumar"),
                       family = "Prayaga",
                       role = "aut",
                       comment = "during 2008"),
                person(given = "Neil",
                       family = "Tiffin",
                       role = "aut"))
NeedsCompilation: yes
Packaged: 2025-03-28 06:57:36 UTC; tomoaki
Author: Joe Conway [aut],
  Dirk Eddelbuettel [aut],
  Tomoaki Nishiyama [aut, cre],
  Sameer Kumar Prayaga [aut] (during 2008),
  Neil Tiffin [aut]
Repository: CRAN
Date/Publication: 2025-03-28 08:00:02 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4.0.2; 2026-02-17 22:21:58 UTC; unix

Package: RPostgreSQL
Version: 0.7-6
Date: 2024-01-11
Title: R Interface to the 'PostgreSQL' Database System
Author: Joe Conway, Dirk Eddelbuettel, Tomoaki Nishiyama, Sameer Kumar Prayaga (during 2008), Neil Tiffin
Maintainer: Tomoaki Nishiyama <tomoakin@staff.kanazawa-u.ac.jp>
Description: Database interface and 'PostgreSQL' driver for 'R'.
 This package provides a Database Interface 'DBI' compliant 
 driver for 'R' to access 'PostgreSQL' database systems.  
 In order to build and install this package from source, 'PostgreSQL' 
 itself must be present your system to provide 'PostgreSQL' functionality 
 via its libraries and header files. These files are provided as
 'postgresql-devel' package under some Linux distributions.
 On 'macOS' and 'Microsoft Windows' system the attached 'libpq' library source will be used.
LazyLoad: true
Depends: R (>= 2.9.0), methods, DBI (>= 0.3)
License: GPL-3 | file LICENSE
Copyright: Authors listed above, PostgreSQL Global Development Group,
        and The Regents of the University of California
Collate: S4R.R zzz.R PostgreSQLSupport.R dbObjectId.R PostgreSQL.R
URL: https://github.com/tomoakin/RPostgreSQL,
        https://cran.r-project.org/package=DBI,
        https://www.postgresql.org
NeedsCompilation: yes
Packaged: 2024-01-11 01:29:20 UTC; tomoaki
Repository: CRAN
Date/Publication: 2024-01-11 06:23:03 UTC

import(methods)
import(DBI)
importFrom("utils", "count.fields", "read.table")

useDynLib(RPostgreSQL, .registration = TRUE)

## Classes
exportClasses(
   dbObjectId,
   PostgreSQLObject,
   PostgreSQLDriver,
   PostgreSQLConnection,
   PostgreSQLResult
)

## Methods/Generics
exportMethods(
   coerce,
   dbApply,
   dbBegin,
   dbCallProc,
   dbClearResult,
   dbColumnInfo,
   dbCommit,
   dbConnect,
   dbDataType,
   dbDisconnect,
#   dbEscapeStrings,
   dbExistsTable,
   dbGetException,
   dbGetInfo,
   dbGetQuery,
   dbGetRowCount,
   dbGetRowsAffected,
   dbGetStatement,
   dbHasCompleted,
   dbListConnections,
   dbListFields,
   dbListResults,
   dbListTables,
#   dbMoreResults,
#   dbNextResult,
   dbReadTable,
   dbRemoveTable,
   dbRollback,
   dbSendQuery,
   dbUnloadDriver,
   dbWriteTable,
   fetch,
   format,
   initialize,
   isSQLKeyword,
   make.db.names,
   show,
   SQLKeywords,
   summary
)

## regular functions (most of these will be made private)
export(
  PostgreSQL,
  postgresqlBuildTableDefinition,
  isPostgresqlIdCurrent,
#  .PostgreSQLKeywords,
  postgresqlInitDriver,
  postgresqlCloseDriver,
  postgresqlDescribeDriver,
  postgresqlDriverInfo,
  postgresqlNewConnection,
  postgresqlCloneConnection,
  postgresqlDescribeConnection,
  postgresqlConnectionInfo,
  postgresqlCloseConnection,
  postgresqlExecStatement,
  postgresqlpqExec,
  postgresqlCopyIn,
  postgresqlCopyInDataframe,
  postgresqlgetResult,
  postgresqlQuickSQL,
  postgresqlDBApply,
  postgresqlFetch,
  postgresqlResultInfo,
  postgresqlDescribeResult,
  postgresqlDescribeFields,
  postgresqlCloseResult,
  postgresqlImportFile,
  postgresqlReadTable,
  postgresqlWriteTable,
  postgresqlEscapeStrings,
  postgresqlEscapeBytea,
  postgresqlUnescapeBytea,
  postgresqlDataType,
  postgresqlQuoteId,
  postgresqlTableRef
)

## constants
#export(
#   CLIENT_LONG_PASSWORD,
#   CLIENT_FOUND_ROWS,
#   CLIENT_LONG_FLAG,
#   CLIENT_CONNECT_WITH_DB,
#   CLIENT_NO_SCHEMA,
#   CLIENT_COMPRESS,
#   CLIENT_ODBC,
#   CLIENT_LOCAL_FILES,
#   CLIENT_IGNORE_SPACE,
#   CLIENT_PROTOCOL_41,
#   CLIENT_INTERACTIVE,
#   CLIENT_SSL,
#   CLIENT_IGNORE_SIGPIPE,
#   CLIENT_TRANSACTIONS,
#   CLIENT_RESERVED,
#   CLIENT_SECURE_CONNECTION,
#   CLIENT_MULTI_STATEMENTS,
#   CLIENT_MULTI_RESULTS
#)

About the RPostgreSQL package:

    Database Interface between R and PostgreSQL

For details, see the Adobe PDF file "DBI.pdf" in the doc folder or see the
documentation using help(PostgreSQL).  Examples provided in the devTests
folder illustrate some of the functionality.

=============================================================================

Basic usage:

  ## initialize the driver to PostgreSQL
  drv <- dbDriver("PostgreSQL")

  ## create a connection to a PostgreSQL server
  con <- dbConnect(drv, user="userName", password="123456",
                   dbname="gsoc", host="10.23.34.23")

  ## run a query, and get the result set as a dataframe
  dFrame <- dbGetQuery(con, "select * from someTable LIMIT 50")

  ## run a query, leave results in the library*
  ## note current implementation does not leave the result in the Server
  ## and there is no benefit using dbSendQuery() fetch() pair other than
  ## compatibility.
  rs <- dbSendQuery(con, "select * from someTable")

  ## fetch up to, say, 50 records
  dFrame <- fetch(rs, n = 50)

  ## close resultSet rs 
  dbClearResult(rs)

  ## close connection con
  dbDisconnect(con)

  ## Unload the driver drv
  dbUnloadDriver(drv)

For a more complete example, refer to the file demo.r in the devTests folder.

=============================================================================

Important Information:

1. The present version of RPostgreSQL can handle only one resultset per connection.
   Thus dbSendQuery/fetch usually do not have speed merit over dbGetQuery.

=============================================================================

Frequently Asked Questions (FAQ):

1.  What is Database Interface(DBI) ?

    The Data Base Interface (DBI) provides a layer of abstraction between R
    and relational databases. All the classes in the DBI package are virtual
    and need to be implemented using the various DBMS libraries. The vendor
    has written some functions for communicating with the database in some
    language like C, compiled the functions and the compiled code is the
    library. We write a C program that calls the functions in the library,
    when it wants to access the database. Every database library is
    different. The names of the functions vary, and the order in
    which you call them varies, and the details of passing queries to the
    functions and getting the data back out will vary. To manage this, DBI
    was extended for individual database back-ends MySQL, SQLite, Oracle,
    PostgreSQL via R packages ROracle, RMySQL, RSQLite and RPostgreSQL. 
    DBI for R-language was initially developed at Bell Labs by David James.

2.  What is RPostgreSQL ?
 
    The RPostgreSQL package provides a glue between the PostgreSQL database
    and the DBI of R. The C programming interface called libpq was used for
    communicating with PostgreSQL. 

3   What is libpq ?

    libpq is the C application programmer’s interface to PostgreSQL. libpq is
    a cross platform library providing set of library functions that allow
    client programs to pass queries to the PostgreSQL backend server and to
    receive the results of these queries.

4.  What about Rdbi/RdbiPgSQL ?

    Rdbi and RdbiPgSQL are a 'fork' of the DBI interface for R. RPostgreSQL follows
    the DBI as do ROracle, RMySQL and RSQLite.


For any queries,suggestions and comments, mail: 
 
    rpostgresql-dev@googlegroups.com

For security issues that should not directly go to public,
you may contact:

    Tomoaki Nishiyama <tomoakin@kenroku.kanazawa-u.ac.jp>
    Neil Tiffin <neilt@neiltiffin.com>
    Joe Conway <mail@joeconway.com>
    Dirk Eddelbuettel <edd@debian.org>

#!/usr/local/bin/R

## Create a database
tempdb <- "tempdbase"
system(paste("createdb", tempdb))

library(RPostgreSQL)
drv <- dbDriver("PostgreSQL")
con <- dbConnect(drv, dbname=tempdb)



## Test the numeric mapping
dbSendQuery(con, "create table testnumeric (intcolumn integer, floatcolumn float);")

i <- as.integer(10)
j <- as.numeric(56.6)

sql <- paste("insert into testnumeric ",
            "values (",i, "," ,j ,") ", sep="")
res <- dbSendQuery(con, sql)


dat <- dbReadTable(con, "testnumeric")
cat("Read Numeric values\n")

## now test the types of the colums we got
stopifnot( class(dat[,1]) == "integer" )
stopifnot( class(dat[,2]) == "numeric" )
cat("GOOD -- all numeric types are as expected\n")

## and test the values
stopifnot( identical( dat[1,1], i))
stopifnot( identical( dat[1,2], j))
cat("GOOD -- all numeric values are as expected\n")




## Test the logical mapping
dbSendQuery(con,"create table testlogical (col1 boolean, col2 boolean)")

i <- as.logical(TRUE)
j <- as.logical(FALSE)

sql <- paste("insert into testlogical ",
            "values (",i, "," ,j ,") ", sep="")
res <- dbSendQuery(con, sql);

dat <- dbReadTable(con, "testlogical")
cat("Read Logical values\n")

## now test the types of the colums we got
stopifnot( class(dat[,1]) == "logical" )
stopifnot( class(dat[,2]) == "logical" )
cat("GOOD -- all logical types are as expected\n")

## and test the values
stopifnot( identical( dat[1,1], i))
stopifnot( identical( dat[1,2], j))
cat("GOOD -- all logical values are as expected\n")




## Test the character mapping
dbSendQuery(con,"create table testchar (code char(3),city varchar(20),country text);")

i <- as.character("IN")
j <- as.character("Hyderabad")
k <- as.character("India")


sql <- paste("insert into testchar ",
            "values ('",i,"' , '",j ,"' , '",k,"') ", sep="")
res <- dbSendQuery(con, sql);

dat <- dbReadTable(con, "testchar")
cat("Read Character values\n")

## now test the types of the colums we got
stopifnot( class(dat[,1]) == "character" )
stopifnot( class(dat[,2]) == "character" )
stopifnot( class(dat[,3]) == "character" )
cat("GOOD -- all character types are as expected\n")


## and test the values
##stopifnot( identical( dat[1,1], i))
stopifnot( identical( dat[1,2], j))
stopifnot( identical( dat[1,3], k))
cat("GOOD -- all character values are as expected\n")


dbDisconnect(con)
dbUnloadDriver(drv)

system(paste("dropdb", tempdb))

cat("DONE\n") 

#!/usr/local/bin/R

cat("Testing the working of Transaction Management\n")

## Create a database
tempdb <- "tempdbase123"
system(paste("createdb", tempdb))

library(RPostgreSQL)
drv <- dbDriver("PostgreSQL")
con <- dbConnect(drv, dbname=tempdb)

## Test the numeric mapping
dbSendQuery(con, "create table book123list (intcolumn integer, floatcolumn float);")

## insert four rows into the table
dbSendQuery(con, "insert into book123list values(12,13.21);")
dbSendQuery(con, "insert into book123list values(50,11.21);")
dbSendQuery(con, "insert into book123list values(100,200.1);")
dbSendQuery(con, "insert into book123list values(5,3.56);")

cat("Table book123list contains the following records\n")
dbGetQuery(con, "select * from book123list")

cat("Test Run 1:\n")

dbBeginTransaction(con);
cat("Begin Transaction\n")
rs <- dbSendQuery(con, "DELETE from book123list WHERE intcolumn >= 50");
dbClearResult(rs);
cat("After Deletion\n");
dbGetQuery(con, "select * from book123list")
dbRollback(con)
cat("After Rolling back\n")

cat("Table book123list contains the following records\n")
dbGetQuery(con, "select * from book123list")

cat("Test Run 2:\n")

dbBeginTransaction(con);
cat("Begin Transaction\n")
dbGetQuery(con, "select * from book123list")[1, ];
rs <- dbSendQuery(con, "DELETE from book123list WHERE intcolumn >= 50");
dbClearResult(rs);
cat("After Deletion\n");
dbGetQuery(con, "select * from book123list")
dbCommit(con);
cat("After commiting the transaction\n")
dbGetQuery(con, "select * from book123list")

dbDisconnect(con)
dbUnloadDriver(drv)

system(paste("dropdb", tempdb))

cat("DONE\n") 

#!/usr/local/bin/R

usePG <- TRUE

if (usePG) {
    cat("Using Pg\n")
    tempdb <- "pgdatetime"
    system(paste("createdb", tempdb))   # create a temp database

    suppressMessages(library(RPostgreSQL))
    drv <- dbDriver("PostgreSQL")
    con <- dbConnect(drv, dbname=tempdb)

} else {
    cat("Using SQLite\n")
    tempdb <- "/tmp/tempdb.sqlite"          # assumed to not exist
    suppressMessages(library(RSQLite))
    drv <- dbDriver("SQLite")
    if (file.exists(tempdb))
        unlink(tempdb)
    con <- dbConnect(drv, tempdb)
}


sql <- "create table foo (i integer, r real, t text)"
res <- dbSendQuery(con, sql)
cat("Created table\n")

i <- as.integer(11)
r <- as.numeric(22.22)
txt <- as.character("blim blom")

sql <- paste("insert into foo ",
             "values (",
             i, ",",
             r, ", '",
             txt, "') ", sep="")
res <- dbSendQuery(con, sql)
cat("Wrote values\n")

df <- dbReadTable(con, "foo")
cat("Read values\n")

## now test the types of the colums we got
stopifnot( class(df[,1]) == "integer" )
stopifnot( class(df[,2]) == "numeric" )
stopifnot( class(df[,3]) == "character" )
cat("GOOD -- all types are as expected\n")

## and test the values
stopifnot( identical( df[1,1], i))
stopifnot( identical( df[1,2], r))
stopifnot( identical( df[1,3], txt))
cat("GOOD -- all values are as expected\n")

if (usePG) {
    dbDisconnect(con)
    dbUnloadDriver(drv)
    system(paste("dropdb", tempdb))
} else {
    dbDisconnect(con)
    dbUnloadDriver(drv)
    unlink(tempdb)
}

cat("DONE\n")

## DEMO for illustrating all the commands in RPostgreSQL package.
## Run this file using 'cat demo.r | R --slave' at the command prompt.

## Create a database
tempdb <- "tempDEMO1dbase"
system(paste("dropdb", tempdb))
system(paste("createdb", tempdb))

cat("\nFor loading the library: library(RPostgreSQL)\n")
library(RPostgreSQL)

cat("\n## Demo for the working of methods in RPostgreSQL\n\n")

cat("##  1.dbDriver(\"driverName\") instantiates the driver object \nEg. drv <- dbDriver(\"PostgreSQL\") \n")
drv <- dbDriver("PostgreSQL")
drv

cat("\n## 2.dbConnect(drv,...) creates and opens a connection to the database implemented by the driver drv. Connection string should be specified with parameters like user, password, dbname, host, port, tty and options. For more details refer to the documentation \nEg. con <- dbConnect(drv,dbname=\"dbName\")\n")
con <- dbConnect(drv, dbname=tempdb)
con
con2<- dbConnect(drv, dbname=tempdb)
con2

cat("\n## 3.dbListConnection(drv, ...): List of connections handled by the driver \nEg. dbListConnections(drv)\n")
dbListConnections(drv)

cat("##  4. dbGetInfo(dbObject, ...) returns information about the dbObject like driver, connection or resultSet \n Eg. dbGetInfo(drv)\n")
dbGetInfo(drv)

cat("## 5. dbSendQuery(con, statement, ...) submits one statement to the database\n Eg. rs <- dbSendQuery(con,\"select * from TableName\")\n")
dbSendQuery(con, "create table book789listCosts (intcolumn integer, floatcolumn float);")
## insert four rows into the table
dbSendQuery(con, "insert into book789listCosts values(12,13.21);")
dbSendQuery(con, "insert into book789listCosts values(50,11.21);")
dbSendQuery(con, "insert into book789listCosts values(100,200.1);")
dbSendQuery(con, "insert into book789listCosts values(5,3.56);")
rs <- dbSendQuery(con, "select * from book789listCosts")

cat("\n## 6. fetch(rs,n, ...) fetches the next n elements from the result set. n=-1 means RETURN ALL ELEMENTS.\n Eg.fetch(rs,n=-1)\n")
fetch(rs,n=-1)

cat("\n## 7. dbGetQuery(con,statement, ...) submits, execute, and extract output in one operation. \nEg.dbGetQuery(con,\"select * from TableName\")\n")
dbGetQuery(con,"select * from book789listCosts")

cat("\n## 8. dbGetException(con, ...) returns the status of the last DBMS statement sent over the connection. \n Eg. dbGetException(con)\n")
dbGetException(con)

cat("\n## 9. dbListResults(con, ...) returns the resultsets active on the given connection. Please note that the current RPostgreSQL package can handle only one resultset per connection (which may change in the future).\n Eg. dbListResults(con)\n")
dbListResults(con)

cat("\n## 10. dbListTables(con, ...) returns the list of tables available on the connection. \n Eg. dbListTables(con)\n")
dbListTables(con)

cat("\n## 11. dbExistsTable(con, TableName, ...) checks whether a particular table exists on the given connection. Returns a logical.\n Eg. dbExistsTable(con,\"name\")\n")
dbExistsTable(con,"name")

cat("\n## 12. dbRemoveTable(con, TableName, ...) removes the specified table on the connection. Returns a logical indicating operation succeeded or not. \n Eg. dbRemoveTable(con,\"name\")\n")

cat("\n## 13. dbListFields(con, TableName, ...) returns the list of column names (fields) in the table.\n Eg. dbListFields(con,\"book789listCosts\")\n")

cat("\n## 14. dbColumnInfo(res, ...) produces a query that describes the output of the query.\n Eg. dbColumnInfo(rs)\n")
dbColumnInfo(rs)

cat("\n## 15. dbReadTable(conn, name, ...) imports the data stored remotely in the table name on connection conn. Use the ﬁeld row.names as the row.names attribute of the output data.frame. Returns a data.frame.\n Eg. dframe <- dbReadTable(con,\"book789listcosts\")\nAfter executing dbReadTable, the dframe contains:\n")
dframe <- dbReadTable(con,"book789listcosts")
dframe

cat("\n## 16. dbWriteTable(conn, name, value, ...) writes the contents of the dataframe \'value\' into the table name specified. Returns a logical indicating whether operation succeeded or not. \n Eg. dbWriteTable(con,\"newTable\",dframe)\n")
dbWriteTable(con,"newTable",dframe)
cat("Checking the contents of newTable: \n Eg. dbGetQuery(con,\"newTable\") \n")
dbGetQuery(con,"select * from newTable")

cat("\n## 17. dbGetStatement(res, ...) returns the DBMS statement associated with the result.\n Eg. dbGetStatement(rs)\n")
dbGetStatement(rs)

cat("\n## 18. dbGetRowsAffected(res, ...) returns the number of rows affected the executed statement. If no rows are affected, \"-1\" is returned. \n Eg. dbGetRowsAffected(rs)\n")
dbGetRowsAffected(rs)

cat("\n## 19. dbHasCompleted(res, ...) returns a logical to indicate whether an operation is completed or not\n Eg. dbHasCompleted(rs)\n")
dbHasCompleted(rs)

cat("\n## 20.dbGetRowCount(res, ...) returns number of rows fetched so far.\n Eg. dbGetRowCount(rs)\n")
dbGetRowCount(rs)

cat("\n## Commands for Transaction Management \n")

cat("\n## 21.dbBeginTransaction begins the PostgreSQL transaction. dbCommit commits the transaction while dbRollback rolls back the transaction. Returns a logical indicating whether the operation succeeded or not.\n")
cat("Eg1. dbBeginTransaction(con) \n")
dbBeginTransaction(con)
cat("Eg1. dbRemoveTable(con,\"newTable\") \n")
dbRemoveTable(con,"newTable")
cat("Eg1. dbExistsTable(con,\"newTable\") \n")
dbExistsTable(con,"newTable")
cat("Eg1. dbRollback(con)\n")
dbRollback(con)
cat("Eg1. dbExistsTable(con,\"newTable\") \n")
dbExistsTable(con,"newTable")
cat("\nEg2. dbBeginTransaction(con) \n")
dbBeginTransaction(con)
cat("Eg2. dbRemoveTable(con,\"newTable\") \n")
dbRemoveTable(con,"newTable")
cat("Eg2. dbExistsTable(con,\"newTable\") \n")
dbExistsTable(con,"newTable")
cat("Eg2. dbCommit(con)\n")
dbCommit(con)
cat("Eg2. dbExistsTable(con,\"newTable\") \n")
dbExistsTable(con,"newTable")


cat("\n## Commands for Freeing the Resources \n")

cat("\n## 22. dbClearResult(rs, ...) flushes any pending data and frees the resources used by result set.\n Eg.dbClearResult(resultSet)\n Cleared the ResultSet :")
dbClearResult(rs)

cat("\n## 23. dbDisconnect(con, ...) closes the connection. \nEg. dbDisconnect(con)")
cat("\nClosed the connections:\n")
dbDisconnect(con)
dbDisconnect(con2)

cat("\n## 24. dbUnloadDriver(drv,...) frees all the resources used by the driver\n Eg. dbUnloadDriver(drv)\n")
cat("Freed the Resources: ")
dbUnloadDriver(drv)

cat("\n")
system(paste("dropdb", tempdb))
cat("Demo Completed\n")

#!/usr/local/bin/R

## assign a basic time type
now <- Sys.time()

basicTypeTests <- function() {

    ## print invokes a conversion to char for 'display' even though the type is really POSIXct
    print(now)
    print(class(now))

    ## we can also convert to char() implicitly
    print(format(now))
    print(class(format(now)))

    ## but what is important is that 'now' is still a time type
    ## that we can 'compute'
    print(now)
    print(now + 60)  ## one minute later
    twomin <- now + 120
    print(as.numeric(now))

    ## and the time is even stored at millisecond granularity!!
    options("digits.secs"=7)		## need to tell R we want sub-second display up to 7 digits
    print(now)				## and now we do
    print(as.numeric(now), digits=16)	## same milli/microseconds here


    ## you can also go the other way and parse a datetime object from a char vector
    then <- strptime("2008-07-01 14:15:16", "%Y-%m-%d %H:%M:%S")
    print(then)
    print(class(then))
    ## and we can convert this from its default 'POSIXlt' ('long type) representation to 'POSIXct' ('compact type')
    then <- as.POSIXct(then)
    print(then)
    print(class(then))
}

dbTypeTests <- function(dateclass="timestamp without time zone") {
    cat("\n\n**** Trying with ", dateclass, "\n")
    tempdb <- "pgdatetime"
    system(paste("createdb", tempdb))   # create a temp database

    stopifnot(require(RPostgreSQL))
    drv <- dbDriver("PostgreSQL")
    con <- dbConnect(drv, dbname=tempdb)

    dbSendQuery(con, paste("create table foo (tt ", dateclass, ", zz integer);", sep=""))
    dbSendQuery(con, "insert into foo values('2008-07-01 14:15:16.123', 1);")

    dbSendQuery(con, paste("insert into foo values('", format(now), "', 2);", sep=""))

    res <- dbReadTable(con, "foo")  ## fails with 'RS-DBI driver warning: (unrecognized PostgreSQL field type 1184 in column 0)'
    print(res)

    ##res <- dbSendQuery(con, "select to_char(tt, 'YYYY-MM-DD HH24:MI:SS.US TZ') as character from foo;")
    res <- dbSendQuery(con, "select tt from foo;")
    data <- fetch(res, n=-1)
    #data <- dbGetQuery(con, "select tt from foo;")
    print(dbColumnInfo(res))
    ##times <- strptime(data[,1], "%Y-%m-%d %H:%M:%OS")
    times <- data[,1]
    print(times)
    print(class(times[1]))

    print(diff(times))	## yes we can compute on date times

    dbDisconnect(con)

    system(paste("dropdb", tempdb))   # create a temp database
}

dbTypeTests()
dbTypeTests("timestamp")
dbTypeTests("timestamp with time zone")
dbTypeTests("date")
#dbTypeTests("time with time zone")
#dbTypeTests("time without time zone")

#!/usr/local/bin/R

suppressMessages({
    library(RODBC)
    library(RPostgreSQL)
})

testTimeData <- function(tt = "date") {

    createTempTable(tt)

    channel <- odbcConnect("beancounter")
    odbcres <- sqlQuery(channel, "select * from timetest")
    close(channel)

    con <- dbConnect( dbDriver("PostgreSQL"), dbname="beancounter")
    pgres <- dbGetQuery(con, "select * from timetest")
    dbDisconnect(con)

    removeTempTable()

    cat("\nFor type='", tt, "'\nODBC returns ", format(odbcres[1,1]), " class ", class(odbcres[1,1]),
        "\nRPostgreSQL returns ", format(pgres[1,1]), " class ", class(pgres[1,1]), "\n", sep="")
}

createTempTable <- function(tt) {
    system(paste("psql beancounter -c \"create table timetest ( d1", tt, ")\" >/dev/null"))
    system("psql beancounter -c \"insert into timetest values( \'now\' )\" >/dev/null")
}

removeTempTable <- function() {
    system("psql beancounter -c \"drop table timetest\" >/dev/null")
}

testTimeData("date")
testTimeData("timestamp with time zone")
testTimeData("timestamp without time zone")
testTimeData("time with time zone")
testTimeData("time without time zone")

## R/S-Plus compatibility
## $Id$

## This package was developed as a part of Summer of Code program organized by Google.
## Thanks to David A. James & Saikat DebRoy, the authors of RMySQL package.
## Code from RMySQL package was reused with the permission from the authors.
## Also Thanks to my GSoC mentor Dirk Eddelbuettel for helping me in the development.


usingR <- function(major=0, minor=0){
    if(is.null(version$language))
        return(FALSE)
    if(version$language!="R")
        return(FALSE)
    version$major>=major && version$minor>=minor
}

## constant holding the appropriate error class returned by try()
if(usingR()){
    ErrorClass <- "try-error"
} else {
    ErrorClass <- "Error"
}

## PostgreSQL.R
## $Id$

## This package was developed as a part of Summer of Code program organized by Google.
## Thanks to David A. James & Saikat DebRoy, the authors of RMySQL package.
## Code from RMySQL package was reused with the permission from the authors.
## Also Thanks to my GSoC mentor Dirk Eddelbuettel for helping me in the development.

##
## Constants
##

##.PostgreSQLRCS <- "$Id: PostgreSQL.R,v 0.1 2008/06/10 14:00:00$"
.PostgreSQLPkgName <- "RPostgreSQL"
.PostgreSQLVersion <- "0.6-1"       ##package.description(.PostgreSQLPkgName, fields = "Version")
.PostgreSQL.NA.string <- "\\N"      ## on input, PostgreSQL interprets \N as NULL (NA)

setOldClass("data.frame")      ## to appease setMethod's signature warnings...

##
## Class: DBIObject
##
setClass("PostgreSQLObject", representation("DBIObject", "dbObjectId", "VIRTUAL"))

##
## Class: dbDriver
##

PostgreSQL <- function(max.con=16, fetch.default.rec = 500, force.reload=FALSE) {
    postgresqlInitDriver(max.con = max.con, fetch.default.rec = fetch.default.rec,
                         force.reload = force.reload)
}

##
## Class: DBIDriver
##
setClass("PostgreSQLDriver", representation("DBIDriver", "PostgreSQLObject"))

## coerce (extract) any PostgreSQLObject into a PostgreSQLDriver
setAs("PostgreSQLObject", "PostgreSQLDriver",
      def = function(from) new("PostgreSQLDriver", Id = as(from, "integer")[1:2])
      )

setMethod("dbUnloadDriver", "PostgreSQLDriver",
          def = function(drv, ...) postgresqlCloseDriver(drv, ...),
          valueClass = "logical"
          )

setMethod("dbGetInfo", "PostgreSQLDriver",
          def = function(dbObj, ...) postgresqlDriverInfo(dbObj, ...)
          )

setMethod("dbListConnections", "PostgreSQLDriver",
          def = function(drv, ...) dbGetInfo(drv, "connectionIds")[[1]]
          )

setMethod("summary", "PostgreSQLDriver",
          def = function(object, ...) postgresqlDescribeDriver(object, ...)
          )

##
## Class: DBIConnection
##
setClass("PostgreSQLConnection", representation("DBIConnection", "PostgreSQLObject"))

setMethod("dbConnect", "PostgreSQLDriver",
          def = function(drv, ...) postgresqlNewConnection(drv, ...),
          valueClass = "PostgreSQLConnection"
          )

setMethod("dbConnect", "character",
          def = function(drv, ...) postgresqlNewConnection(dbDriver(drv), ...),
          valueClass = "PostgreSQLConnection"
          )

## clone a connection
setMethod("dbConnect", "PostgreSQLConnection",
          def = function(drv, ...) postgresqlCloneConnection(drv, ...),
          valueClass = "PostgreSQLConnection"
          )

setMethod("dbDisconnect", "PostgreSQLConnection",
          def = function(conn, ...) postgresqlCloseConnection(conn, ...),
          valueClass = "logical"
          )

setGeneric("dbEscapeStrings", def = function(conn, string, ...) standardGeneric("dbEscapeStrings"))
setMethod("dbEscapeStrings",
          signature(conn="PostgreSQLConnection", string="character"),
          def = function(conn, string, ...) postgresqlEscapeStrings(conn, string, ...),
          valueClass = "character"
          )

setMethod("dbSendQuery",
          signature(conn = "PostgreSQLConnection", statement = "character"),
          def = function(conn, statement,...) postgresqlExecStatement(conn, statement,...),
          valueClass = "PostgreSQLResult"
          )


setMethod("dbGetQuery",
          signature(conn = "PostgreSQLConnection", statement = "character"),
          def = function(conn, statement, ...) postgresqlQuickSQL(conn, statement, ...)
          )

setMethod("dbGetException", "PostgreSQLConnection",
          def = function(conn, ...){
              if(!isPostgresqlIdCurrent(conn))
                  stop(paste("expired", class(conn)))
              .Call(RS_PostgreSQL_getException, as(conn, "integer"))
          },
          valueClass = "list"
          )

setMethod("dbGetInfo", "PostgreSQLConnection",
          def = function(dbObj, ...) postgresqlConnectionInfo(dbObj, ...)
          )

setMethod("dbListResults", "PostgreSQLConnection",
          def = function(conn, ...) dbGetInfo(conn, "rsId")[[1]]
          )

setMethod("summary", "PostgreSQLConnection",
          def = function(object, ...) postgresqlDescribeConnection(object, ...)
          )

## convenience methods
setMethod("dbListTables", "PostgreSQLConnection",
          def = function(conn, ...){
              out <- dbGetQuery(conn,
                                paste("select tablename from pg_tables where schemaname !='information_schema'",
                                      "and schemaname !='pg_catalog'", ...))
              if (is.null(out) || nrow(out) == 0)
                  out <- character(0)
              else
                  out <- out[, 1]
              out
          },
          valueClass = "character"
          )

setMethod("dbReadTable", signature(conn="PostgreSQLConnection", name="character"),
          def = function(conn, name, ...) postgresqlReadTable(conn, name, ...),
          valueClass = "data.frame"
          )

setMethod("dbWriteTable",
          signature(conn="PostgreSQLConnection", name="character", value="data.frame"),
          def = function(conn, name, value, ...){
              postgresqlWriteTable(conn, name, value, ...)
          },
          valueClass = "logical"
          )

## write table from filename (TODO: connections)
setMethod("dbWriteTable",
          signature(conn="PostgreSQLConnection", name="character", value="character"),
          def = function(conn, name, value, ...){
              postgresqlImportFile(conn, name, value, ...)
          },
          valueClass = "logical"
          )

setMethod("dbExistsTable",
          signature(conn="PostgreSQLConnection", name="character"),
          def = function(conn, name, ...){
              qlength <- length(name)
              if(qlength == 1){
              currentschema <- dbGetQuery(conn, "SELECT current_schema()")
              res <- dbGetQuery(conn,
                  paste("select tablename from pg_tables where ",
                  "schemaname !='information_schema' and schemaname !='pg_catalog' ",
                  "and schemaname='", postgresqlEscapeStrings(conn, currentschema[[1]]), "' ",
                  "and tablename='", postgresqlEscapeStrings(conn, name), "'", sep=""))
              }
              else{
                  if(qlength == 2){
                  res <- dbGetQuery(conn,
                      paste("select tablename from pg_tables where ",
                      "schemaname !='information_schema' and schemaname !='pg_catalog' ",
                      "and schemaname='", postgresqlEscapeStrings(conn, name[1]), "' ",
                      "and tablename='", postgresqlEscapeStrings(conn, name[2]), "'", sep=""))
                  }
              }
              return(as.logical(dim(res)[1]))
          },
          valueClass = "logical"
          )

setMethod("dbRemoveTable",
          signature(conn="PostgreSQLConnection", name="character"),
          def = function(conn, name, ...){
              if(dbExistsTable(conn, name)){
                  rc <- try(dbGetQuery(conn, paste("DROP TABLE", postgresqlTableRef(name))))
                  !inherits(rc, ErrorClass)
              }
              else FALSE
          },
          valueClass = "logical"
          )

setMethod("dbBegin",
          signature(conn="PostgreSQLConnection"),
          def = function(conn,  ...){
              rc <- try(dbGetQuery(conn, "BEGIN"))
              !inherits(rc, ErrorClass)
          },
          valueClass = "logical"
          )
setMethod("dbCommit",
          signature(conn="PostgreSQLConnection"),
          def = function(conn,  ...){
              rc <- try(dbGetQuery(conn, "COMMIT"))
              !inherits(rc, ErrorClass)
          },
          valueClass = "logical"
          )
setMethod("dbRollback",
          signature(conn="PostgreSQLConnection"),
          def = function(conn,  ...){
              rc <- try(dbGetQuery(conn, "ROLLBACK"))
              !inherits(rc, ErrorClass)
          },
          valueClass = "logical"
          )

## return field names (no metadata)
setMethod("dbListFields",
          signature(conn="PostgreSQLConnection", name="character"),
          def = function(conn, name, ...){
              qlength <- length(name)
              if(qlength == 1){
              currentschema <- dbGetQuery(conn, "SELECT current_schema()")
              flds <- dbGetQuery(conn,
                  paste0("select a.attname from pg_attribute a, pg_class c, pg_tables t, pg_namespace nsp ",
                  "where a.attrelid = c.oid and c.relname = tablename and c.relnamespace = nsp.oid and ",
                  "a.attnum > 0 and not a.attisdropped and ",
                  "nspname = current_schema() and ",
                  "schemaname = nspname and ",
                  "tablename = '", postgresqlEscapeStrings(conn, name), "'"))[,1]
              }
              else{
                  if(qlength == 2){
                  flds <- dbGetQuery(conn,
                      paste0("select a.attname from pg_attribute a, pg_class c, pg_tables t, pg_namespace nsp ",
                      "where a.attrelid = c.oid and c.relname = t.tablename and c.relnamespace = nsp.oid and ",
                      "a.attnum > 0 and not a.attisdropped and ",
                      "nspname = schemaname and ",
                      "schemaname = '", postgresqlEscapeStrings(conn, name[1]), "' and ",
                      "tablename = '", postgresqlEscapeStrings(conn, name[2]), "'"))[,1]
                  }
              }

              if(length(flds)==0)
                  flds <- character()
              flds
          },
          valueClass = "character"
          )


setMethod("dbCallProc", "PostgreSQLConnection",
          def = function(conn, ...) .NotYetImplemented()
          )

setMethod("dbCommit", "PostgreSQLConnection",
          def = function(conn, ...) postgresqlTransactionStatement(conn, "COMMIT")
          )

setMethod("dbRollback", "PostgreSQLConnection",
          def = function(conn, ...) {
              rsList <- dbListResults(conn)
              if (length(rsList))
                  dbClearResult(rsList[[1]])
              postgresqlTransactionStatement(conn, "ROLLBACK")
          }
          )


##
## Class: DBIResult
##
setClass("PostgreSQLResult", representation("DBIResult", "PostgreSQLObject"))

setAs("PostgreSQLResult", "PostgreSQLConnection",
      def = function(from) new("PostgreSQLConnection", Id = as(from, "integer")[1:3])
      )
setAs("PostgreSQLResult", "PostgreSQLDriver",
      def = function(from) new("PostgreSQLDriver", Id = as(from, "integer")[1:2])
      )

setMethod("dbClearResult", "PostgreSQLResult",
          def = function(res, ...) postgresqlCloseResult(res, ...),
          valueClass = "logical"
          )

setMethod("fetch", signature(res="PostgreSQLResult", n="numeric"),
          def = function(res, n, ...){
              out <- postgresqlFetch(res, n, ...)
              if(is.null(out))
                  out <- data.frame(out)
              out
          },
          valueClass = "data.frame"
          )

setMethod("fetch",
          signature(res="PostgreSQLResult", n="missing"),
          def = function(res, n, ...){
              out <-  postgresqlFetch(res, n=0, ...)
              if(is.null(out))
                  out <- data.frame(out)
              out
          },
          valueClass = "data.frame"
          )

setMethod("dbGetInfo", "PostgreSQLResult",
          def = function(dbObj, ...) postgresqlResultInfo(dbObj, ...),
          valueClass = "list"
          )

setMethod("dbGetStatement", "PostgreSQLResult",
          def = function(res, ...){
              st <-  dbGetInfo(res, "statement")[[1]]
              if(is.null(st))
                  st <- character()
              st
          },
          valueClass = "character"
          )

setMethod("dbListFields",
          signature(conn="PostgreSQLResult", name="missing"),
          def = function(conn, name, ...){
              flds <- dbGetInfo(conn, "fields")$fields$name
              if(is.null(flds))
                  flds <- character()
              flds
          },
          valueClass = "character"
          )

setMethod("dbColumnInfo", "PostgreSQLResult",
          def = function(res, ...) postgresqlDescribeFields(res, ...),
          valueClass = "data.frame"
          )

setMethod("dbGetRowsAffected", "PostgreSQLResult",
          def = function(res, ...) dbGetInfo(res, "rowsAffected")[[1]],
          valueClass = "numeric"
          )

setMethod("dbGetRowCount", "PostgreSQLResult",
          def = function(res, ...) dbGetInfo(res, "rowCount")[[1]],
          valueClass = "numeric"
          )

setMethod("dbHasCompleted", "PostgreSQLResult",
          def = function(res, ...) dbGetInfo(res, "completed")[[1]] == 1,
          valueClass = "logical"
          )

setMethod("dbGetException", "PostgreSQLResult",
          def = function(conn, ...){
              id <- as(conn, "integer")[1:2]
              .Call(RS_PostgreSQL_getException, id)
          },
          valueClass = "list"    ## TODO: should be a DBIException?
          )

setMethod("summary", "PostgreSQLResult",
          def = function(object, ...) postgresqlDescribeResult(object, ...)
          )



setMethod("dbDataType",
          signature(dbObj = "PostgreSQLObject", obj = "ANY"),
          def = function(dbObj, obj, ...) postgresqlDataType(obj, ...),
          valueClass = "character"
          )


## MODIFIED : -- sameer
setMethod("make.db.names",
          signature(dbObj="PostgreSQLObject", snames = "character"),
          def = function(dbObj, snames,keywords,unique, allow.keywords,...){
              postgresqlQuoteId(snames)
          },
          valueClass = "character"
          )

setMethod("SQLKeywords", "PostgreSQLObject",
          def = function(dbObj, ...) .PostgreSQLKeywords,
          valueClass = "character"
          )

setMethod("isSQLKeyword",
          signature(dbObj="PostgreSQLObject", name="character"),
          def = function(dbObj, name,keywords,case, ...){
              isSQLKeyword.default(name, keywords = .PostgreSQLKeywords)
          },
          valueClass = "character"
          )
## extension to the DBI 0.1-4
setGeneric("dbApply", def = function(res, ...) standardGeneric("dbApply"))
setMethod("dbApply", "PostgreSQLResult",
          def = function(res, ...)  postgresqlDBApply(res, ...),
          )

## zzz.R
## $Id$

## This package was developed as a part of Summer of Code program organized by Google.
## Thanks to David A. James & Saikat DebRoy, the authors of RMySQL package.
## Code from RMySQL package was reused with the permission from the authors.
## Also Thanks to my GSoC mentor Dirk Eddelbuettel for helping me in the development.

.onLoad <- function(lib, pkg) {
    library.dynam("RPostgreSQL", pkg, lib)
}

## dbExists test with schema name: reported as Issue 3 at
## http://code.google.com/p/rpostgresql/issues/detail?id=3
## and based on an earlier email by Prasenjit Kapat
##
## Assumes that
##  a) PostgreSQL is running, and
##  b) the current user can connect
## both of which are not viable for release but suitable while we test
##
## Dirk Eddelbuettel, 10 Sep 2009

## only run this if this env.var is set correctly
if (Sys.getenv("POSTGRES_USER") != "" & Sys.getenv("POSTGRES_HOST") != "" & Sys.getenv("POSTGRES_DATABASE") != "") {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))
    stopifnot(require(datasets))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))

    if (dbExistsTable(con, "rockdata")) {
        print("Removing rockdata\n")
        dbRemoveTable(con, "rockdata")
    }

    dbWriteTable(con, "rockdata", rock)

    ## run a simple dbExists
    cat("Does rockdata exist? ")
    print(res <- dbExistsTable(con, "rockdata"))

    ## this should return TRUE but did not -- does now after patch
    cat("Does public.rockdata exist? ")
    print(res <- dbExistsTable(con, "public.rockdata"))

    ## cleanup
    if (dbExistsTable(con, "rockdata")) {
        print("Removing rockdata\n")
        dbRemoveTable(con, "rockdata")
    }

    ## and disconnect
    dbDisconnect(con)
}

## dbWriteTable test
##
## Assumes that
##  a) PostgreSQL is running, and
##  b) the current user can connect
## both of which are not viable for release but suitable while we test
##
## Dirk Eddelbuettel, 10 Sep 2009

## only run this if this env.var is set correctly
if (Sys.getenv("POSTGRES_USER") != "" & Sys.getenv("POSTGRES_HOST") != "" & Sys.getenv("POSTGRES_DATABASE") != "") {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))
    stopifnot(require(datasets))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")

    ## create two independent connections
    con1 <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))

    con2 <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))


    if (dbExistsTable(con1, "rockdata")) {
        cat("Removing rockdata\n")
        dbRemoveTable(con1, "rockdata")
    }

    cat("begin transaction in con1\n")
    dbGetQuery(con1, "BEGIN TRANSACTION")
    cat("create table rockdata in con1\n")
    dbWriteTable(con1, "rockdata", rock)
    if (dbExistsTable(con1, "rockdata")) {
        cat("PASS rockdata is visible through con1\n")
    }else{
        cat("FAIL rockdata is invisible through con1\n")
    }
    if (dbExistsTable(con2, "rockdata")) {
        cat("FAIL rockdata is visible through con2\n")
    }else{
        cat("PASS rockdata is invisible through con2\n")
    }
    cat("commit in con1\n")
    dbCommit(con1)
    if (dbExistsTable(con2, "rockdata")) {
        cat("PASS rockdata is visible through con2\n")
    }else{
        cat("FAIL rockdata is invisible through con2\n")
    }

    cat("remove the table from con1\n")
    dbRemoveTable(con1, "rockdata")

    if (dbExistsTable(con2, "rockdata")) {
        cat("FAIL rockdata is visible through con2\n")
    }else{
        cat("PASS rockdata is invisible through con2\n")
    }

    cat("begin transaction in con1\n")
    dbGetQuery(con1, "BEGIN TRANSACTION")
    cat("create table rockdata in con1\n")
    dbWriteTable(con1, "rockdata", rock)
    if (dbExistsTable(con1, "rockdata")) {
        cat("PASS rockdata is visible through con1\n")
    }else{
        cat("FAIL rockdata is invisible through con1\n")
    }
    cat("RollBack con1\n")
    dbRollback(con1)
    if (dbExistsTable(con1, "rockdata")) {
        cat("FAIL rockdata is visible through con1\n")
    }else{
        cat("PASS rockdata is invisible through con1\n")
    }

    ## and disconnect
    dbDisconnect(con2)
    dbDisconnect(con1)
}

## dbWriteTable test
##
## Assumes that
##  a) PostgreSQL is running, and
##  b) the current user can connect
## both of which are not viable for release but suitable while we test
##
## Dirk Eddelbuettel, 10 Sep 2009

## only run this if this env.var is set correctly
if (Sys.getenv("POSTGRES_USER") != "" & Sys.getenv("POSTGRES_HOST") != "" & Sys.getenv("POSTGRES_DATABASE") != "") {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))
    stopifnot(require(datasets))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")

    ## create two independent connections
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))

    if (dbExistsTable(con, "tmptest")) {
        cat("Removing tmptest\n")
        dbRemoveTable(con, "tmptest")
    }
# create temporary table (as a copy of any existing one)
#    dbGetQuery(con, "BEGIN TRANSACTION")
    cat("create temp table tmptest with dbGetQuery\n")
    dbGetQuery(con, "CREATE TEMP TABLE tmptest (f1 int)")

# query temp table
    rs<-dbGetQuery(con, "SELECT * from tmptest")
    print(rs)
    
    cat("create temp table tmptest with dbSendQuery\n")
    dbSendQuery(con, "CREATE TEMP TABLE tmptest2(f2 int)")

# query temp table
    rs2<-dbGetQuery(con, "SELECT * from tmptest2")
    print(rs2)
    ## and disconnect
    dbDisconnect(con)

    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))

    if (dbExistsTable(con, "tmptest")) {
        cat("FAIL tmptest persisted after disconnection\n")
        cat("Removing tmptest\n")
        dbRemoveTable(con, "tmptest")
    }else{
        cat("PASS tmptest disappeared after disconnection\n")
    }
    dbDisconnect(con)
}

## dbWriteTable test
##
## Assumes that
##  a) PostgreSQL is running, and
##  b) the current user can connect
## both of which are not viable for release but suitable while we test
##
## Dirk Eddelbuettel, 10 Sep 2009

## only run this if this env.var is set correctly
if (Sys.getenv("POSTGRES_USER") != "" & Sys.getenv("POSTGRES_HOST") != "" & Sys.getenv("POSTGRES_DATABASE") != "") {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))


    #  create a table
    res <- dbGetQuery(con, "CREATE TABLE aa (pk integer, v1 float not null, v2 float)" )

    ## run a simple query and show the query result
    res <- dbGetQuery(con, "INSERT INTO aa VALUES(3, 2, NULL)" )
    res <- dbSendQuery(con, "select pk, v1, v2, v1+v2 from aa")
    cat("dbColumnInfo\n")
    print(dbColumnInfo(res))
    print(dbColumnInfo(res))
    cat("SELECT result\n")
    df <- fetch(res, n=-1)
    print(df)

    ## cleanup
    cat("Removing \"AA\"\n")
    dbRemoveTable(con, "aa")
    ## and disconnect
    dbDisconnect(con)
}

## dbGetQuery test with optional parameters
##
## Assumes that
##  a) PostgreSQL is running, and
##  b) the current user can connect
## both of which are not viable for release but suitable while we test

## only run this if this env.var is set correctly
if (Sys.getenv("POSTGRES_USER") != "" & Sys.getenv("POSTGRES_HOST") != "" & Sys.getenv("POSTGRES_DATABASE") != "") {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))
    stopifnot(require(datasets))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))


    if (dbExistsTable(con, "rockdata")) {
        print("Removing rockdata\n")
        dbRemoveTable(con, "rockdata")
    }

    dbWriteTable(con, "rockdata", rock)

    ## run a simple query and show the query result
    res <- dbGetQuery(con, "SELECT * FROM rockdata WHERE peri > $1 LIMIT 10", 4000)
    print(res)
    res <- dbGetQuery(con, "SELECT * FROM rockdata WHERE peri > $1 AND shape < $2 LIMIT $3", c(4000, 0.2, 10))
    print(res)


    ## cleanup
    if (dbExistsTable(con, "rockdata")) {
        print("Removing rockdata\n")
        dbRemoveTable(con, "rockdata")
    }

    ## and disconnect
    dbDisconnect(con)
}

## PostgreSQLSupport.R
## $Id$

## This package was developed as a part of Summer of Code program organized by Google.
## Thanks to David A. James & Saikat DebRoy, the authors of RMySQL package.
## Code from RMySQL package was reused with the permission from the authors.
## Also Thanks to my GSoC mentor Dirk Eddelbuettel for helping me in the development.

## create a PostgreSQL database connection manager.  By default we allow
## up to "max.con" connections and single fetches of up to "fetch.default.rec"
## records.  These settings may be changed by re-loading the driver
## using the "force.reload" = T flag (note that this will close all
## currently open connections).
## Returns an object of class "PostgreSQLManger".
## Note: This class is a singleton.
postgresqlInitDriver <- function(max.con=16, fetch.default.rec = 500, force.reload=FALSE) {
   if(fetch.default.rec<=0)
       stop("default num of records per fetch must be positive")
   config.params <- as.integer(c(max.con, fetch.default.rec))
   force <- as.logical(force.reload)
   drvId <- .Call(RS_PostgreSQL_init, config.params, force)
   new("PostgreSQLDriver", Id = drvId)
}

postgresqlCloseDriver <- function(drv, ...) {
   if(!isPostgresqlIdCurrent(drv))
      return(TRUE)
   drvId <- as(drv, "integer")
   .Call(RS_PostgreSQL_closeManager, drvId)
}


## Print out nicely a brief description of the connection Driver
postgresqlDescribeDriver <- function(obj, verbose = FALSE, ...) {
   info <- dbGetInfo(obj)
   print(obj)
   cat("  Driver name: ", info$drvName, "\n")
   cat("  Max  connections:", info$length, "\n")
   cat("  Conn. processed:", info$counter, "\n")
   cat("  Default records per fetch:", info$"fetch_default_rec", "\n")
   if(verbose){
      cat("  DBI API version: ", dbGetDBIVersion(), "\n")
      ##  cat("  PostgreSQL client version: ", info$clientVersion, "\n")  Feature absent
   }
   cat("  Open connections:", info$"num_con", "\n")
   if(verbose && !is.null(info$connectionIds)){
      for(i in seq_along(info$connectionIds)){
         cat("   ", i, " ")
         print(info$connectionIds[[i]])
      }
   }
   invisible(NULL)
}

postgresqlDriverInfo <- function(obj, what="", ...) {
    if(!isPostgresqlIdCurrent(obj))
        stop(paste("expired", class(obj)))
    drvId <- as(obj, "integer")
    info <- .Call(RS_PostgreSQL_managerInfo, drvId)
    ## replace drv/connection id w. actual drv/connection objects
    conObjs <- vector("list", length = info$"num_con")
    ids <- info$connectionIds
    for(i in seq_along(ids))
        conObjs[[i]] <- new("PostgreSQLConnection", Id = c(drvId, ids[i]))
    info$connectionIds <- conObjs
    info$managerId <- new("PostgreSQLDriver", Id = drvId)
    if(!missing(what))
        info[what]
    else
        info
}

## All NULL values are changed to empty strings (""). 
## These are treated as default values.
postgresqlNewConnection <- function(drv, user = "", password = "",
                                    host = "", dbname = "",
                                    port = "", tty = "", options = "", forceISOdate=TRUE) {
    if(!isPostgresqlIdCurrent(drv))
        stop("expired manager")
    if(is.null(user))
        user=""
    if(is.null(password))
        password=""
    if(is.null(host))
        host=""
    if(is.null(dbname))
        dbname=""
    if(is.null(port))
        port=""
    if(is.null(tty))
        tty=""
    if(is.null(options))
        options=""

    con.params <- as.character(c(user, password, host,
                                 dbname, port,
                                 tty, options))

    drvId <- as(drv, "integer")
    conId <- .Call(RS_PostgreSQL_newConnection, drvId, con.params)
    con <- new("PostgreSQLConnection", Id = conId)
    if(forceISOdate){
      dbGetQuery(con, "set datestyle to ISO")      
    }
    con
}

postgresqlCloneConnection <- function(con, ...) {
    if(!isPostgresqlIdCurrent(con))
        stop(paste("expired", class(con)))
    conId <- as(con, "integer")
    newId <- .Call(RS_PostgreSQL_cloneConnection, conId)
    new("PostgreSQLConnection", Id = newId)
}

postgresqlDescribeConnection <- function(obj, verbose = FALSE, ...) {
    info <- dbGetInfo(obj)
    print(obj)
    cat("  User:", info$user, "\n")
    cat("  Host:", info$host, "\n")
    cat("  Dbname:", info$dbname, "\n")

    ## cat("  Connection type:", info$conType, "\n")     function absent

    if(verbose){
        cat("  PostgreSQL server version: ", info$serverVersion, "\n")
        ##     cat("  PostgreSQL client version: ",
        ##         dbGetInfo(as(obj, "PostgreSQLDriver"), what="clientVersion")[[1]], "\n")           feature absent
        cat("  PostgreSQL protocol version: ", info$protocolVersion, "\n")
        cat("  PostgreSQL server thread id: ", info$threadId, "\n")
    }
    if (length(info$rsId)>0) {
        for (i in seq_along(info$rsId)) {
            cat("   ", i, " ")
            print(info$rsId[[i]])
        }
    } else {
        cat("  No resultSet available\n")
    }
    invisible(NULL)
}

postgresqlCloseConnection <- function(con, ...) {
    if(!isPostgresqlIdCurrent(con))
        return(TRUE)
    rs <- dbListResults(con)
    if(length(rs)>0){
        if(dbHasCompleted(rs[[1]]))
            dbClearResult(rs[[1]])
        else
            stop("connection has pending rows (close open results set first)")
    }
    conId <- as(con, "integer")
    .Call(RS_PostgreSQL_closeConnection, conId)
}

postgresqlConnectionInfo <- function(obj, what="", ...) {
    if(!isPostgresqlIdCurrent(obj))
        stop(paste("expired", class(obj), deparse(substitute(obj))))
    id <- as(obj, "integer")
    info <- .Call(RS_PostgreSQL_connectionInfo, id)
    rsId <- vector("list", length = length(info$rsId))
    for(i in seq_along(info$rsId))
        rsId[[i]] <- new("PostgreSQLResult", Id = c(id, info$rsId[i]))
    info$rsId <- rsId
    if(!missing(what))
        info[what]
    else
        info
}

## checks for any open resultsets, and closes them if completed.
## the statement is then executed on the connection, and returns
## whether it executed without an error or not.
postgresqlTransactionStatement <- function(con, statement) {
    ## are there resultSets pending on con?
    if(length(dbListResults(con)) > 0){
        res <- dbListResults(con)[[1]]
        if(!dbHasCompleted(res)){
            stop("connection with pending rows, close resultSet before continuing")
        }
        dbClearResult(res)
    }

    rc <- try(dbGetQuery(con, statement))
    !inherits(rc, ErrorClass)
}


## submits the sql statement to PostgreSQL and creates a
## dbResult object if the SQL operation does not produce
## output, otherwise it produces a resultSet that can
## be used for fetching rows.
postgresqlExecStatement <- function(con, statement, params, ...) {
    if(!isPostgresqlIdCurrent(con))
        stop(paste("expired", class(con)))
    conId <- as(con, "integer")
    statement <- as(statement, "character")
    if(missing(params)){
        rsId <- .Call(RS_PostgreSQL_exec, conId, statement)
    }else{
        rsId <- .External(RS_PostgreSQL_pqexecparams, conId, statement, as(params, "character"), ...)
    }
    new("PostgreSQLResult", Id = rsId)
}

postgresqlEscapeStrings <- function(con, preescapedstring) {
    conId <- as(con, "integer")
    preescapedstring <- as(preescapedstring, "character")
    escapedstring <- .Call(RS_PostgreSQL_escape, conId, preescapedstring)
    return(escapedstring)
}
postgresqlEscapeBytea <- function(con, raw_data) {
    conId <- as(con, "integer")
    raw_data <- as(raw_data, "raw")
    escapedstring <- .Call(RS_PostgreSQL_escape_bytea, conId, raw_data)
    return(escapedstring)
}
postgresqlUnescapeBytea <- function(escapedbytea) {
    escapedbytea <- as(escapedbytea, "character")
    raw_data <- .Call(RS_PostgreSQL_unescape_bytea, escapedbytea)
    return(raw_data)
}
postgresqlpqExec <- function(con, statement) {
    if(!isPostgresqlIdCurrent(con))
        stop(paste("expired", class(con)))
    conId <- as(con, "integer")
    statement <- as(statement, "character")
    .Call(RS_PostgreSQL_pqexec, conId, statement)
}
postgresqlpqExecParams <- function(con, statement, ...) {
    if(!isPostgresqlIdCurrent(con))
        stop(paste("expired", class(con)))
    conId <- as(con, "integer")
    statement <- as(statement, "character")
    .External(RS_PostgreSQL_pqexecparams, conId, statement, ...)
}

postgresqlCopyIn <- function(con, filename) {
    if(!isPostgresqlIdCurrent(con))
        stop(paste("expired", class(con)))
    conId <- as(con, "integer")
    filename <- as(filename, "character")
    .Call(RS_PostgreSQL_CopyIn, conId, filename)
}
postgresqlCopyInDataframe <- function(con, dataframe) {
    if(!isPostgresqlIdCurrent(con))
        stop(paste("expired", class(con)))
    conId <- as(con, "integer")
    nrow <- nrow(dataframe)
    p <- ncol(dataframe)
    .Call(RS_PostgreSQL_CopyInDataframe, conId, dataframe, nrow, p)
}
postgresqlgetResult <- function(con) {
    if(!isPostgresqlIdCurrent(con))
        stop(paste("expired", class(con)))
    conId <- as(con, "integer")
    rsId <- .Call(RS_PostgreSQL_getResult, conId)
    new("PostgreSQLResult", Id = rsId)
}


## helper function: it exec's *and* retrieves a statement. It should
## be named somehting else.
postgresqlQuickSQL <- function(con, statement, ...) {
    if(!isPostgresqlIdCurrent(con))
        stop(paste("expired", class(con)))
    rsList <- dbListResults(con)
    if (length(rsList)>0){  # clear results
        dbClearResult(rsList[[1]])
    }
    rs <- try(dbSendQuery(con, statement, ...))
    if (inherits(rs, ErrorClass)){
        warning("Could not create execute: ", statement)
        return(NULL)
    }
    if(dbHasCompleted(rs)){
        dbClearResult(rs)            ## no records to fetch, we're done
        invisible()
        return(data.frame()) # data.frame of 0 row 0 columns
    }
    res <- fetch(rs, n = -1)
    if(dbHasCompleted(rs))
        dbClearResult(rs)
    else
        warning("pending rows")  # shouldn't occur...
    res
}

postgresqlDescribeFields <- function(res, ...) {
    flds <- dbGetInfo(res, "fieldDescription")[[1]][[1]]
    if(!is.null(flds)){
        flds$Sclass <- .Call(RS_DBI_SclassNames, flds$Sclass)

        ## -------
        ## This is actually a bug introduced deliberately. In dbGetInfo, it displays the Sclass for Date/Time datatypes in Pg
        ## as character. But in dbColumnInfo, it displays it as 'POSIXct'. This is because there is no
        ## datatype corresponding to Date/Time defined in R-defines.h & R-internals.h
        for(i in 1:length(flds$type)) {
            if(flds$type[[i]] == 1114) {
                flds$Sclass[[i]] = "POSIXct";
            } else if(flds$type[[i]] == 1082) {
                flds$Sclass[[i]] = "Date";
            } else if(flds$type[[i]] == 1184) {
   		flds$Sclass[[i]] = "POSIXct";
            }
        }
        ## -------

        flds$type <- .Call(RS_PostgreSQL_typeNames, as.integer(flds$type))
        ## no factors
        structure(flds, row.names = paste(seq_along(flds$type)),
                  class = "data.frame")
    }
    else data.frame(flds)
}

## (Experimental)
## This function is meant to handle somewhat gracefully(?) large amounts
## of data from the DBMS by bringing into R manageable chunks (about
## batchSize records at a time, but not more than maxBatch); the idea
## is that the data from individual groups can be handled by R, but
## not all the groups at the same time.
##
## dbApply apply functions to groups of rows coming from a remote
## database resultSet upon the following fetching events:
##   begin         (prior to fetching the first record)
##   group.begin   (the record just fetched begins a new group)
##   new_record    (a new record just fetched)
##   group.end     (the record just fetched ends the current group)
##   end           (the record just fetched is the very last record)
##
## The "begin", "begin.group", etc., specify R functions to be
## invoked upon the corresponding events.  (For compatibility
## with other apply functions the arg FUN is used to specify the
## most common case where we only specify the "group.end" event.)
##
## The following describes the exact order and form of invocation for the
## various callbacks in the underlying  C code.  All callback function
## (except FUN) are optional.
##  begin()
##    group.begin(group.name)
##    new.record(df.record)
##    FUN(df.group, group.name)   (aka group.end)
##  end()
##
## TODO: (1) add argument output=F/T to suppress the creation of
##           an expensive(?) output list.
##       (2) allow INDEX to be a list as in tapply()
##       (3) should we implement a simplify argument, as in sapply()?
##       (4) should report (instead of just warning) when we're forced
##           to handle partial groups (groups larger than maxBatch).
##       (5) extend to the case where even individual groups are too
##           big for R (as in incrementatl quantiles).
##       (6) Highly R-dependent, not sure yet how to port it to S-plus.
##
postgresqlDBApply <- function(res, INDEX, FUN = stop("must specify FUN"),
                              begin = NULL,
                              group.begin =  NULL,
                              new.record = NULL,
                              end = NULL,
                              batchSize = 100, maxBatch = 1e6,
                              ..., simplify = TRUE) {
    if(dbHasCompleted(res))
        stop("result set has completed")
    if(is.character(INDEX)){
        flds <- tolower(as.character(dbColumnInfo(res)$name))
        INDEX <- match(tolower(INDEX[1]), flds, 0)
    }
    if(INDEX<1)
        stop(paste("INDEX field", INDEX, "not in result set"))

    null.or.fun <- function(fun) { # get fun obj, but a NULL is ok
        if(is.null(fun))
            fun
        else
            match.fun(fun)
    }
    begin <- null.or.fun(begin)
    group.begin <- null.or.fun(group.begin)
    group.end <- null.or.fun(FUN)     ## probably this is the most important
    end <- null.or.fun(end)
    new.record <- null.or.fun(new.record)
    rsId <- as(res, "integer")
    con <- as(res, "PostgreSQLConnection")
    on.exit({
        rc <- dbGetException(con)
        if(!is.null(rc$errorNum) && rc$errorNum!=0)
            cat("dbApply aborted with PostgreSQL error ", rc$errorNum,
                " (", rc$errorMsg, ")\n", sep = "")

    })
    ## BEGIN event handler (re-entrant, only prior to reading first row)
    if(!is.null(begin) && dbGetRowCount(res)==0)
        begin()
    rho <- environment()
    funs <- list(begin = begin, end = end,
                 group.begin = group.begin,
                 group.end = group.end, new.record = new.record)
    out <- .Call(RS_PostgreSQL_dbApply,
                 rs = rsId,
                 INDEX = as.integer(INDEX-1),
                 funs, rho, as.integer(batchSize), as.integer(maxBatch))
    if(!is.null(end) && dbHasCompleted(res))
        end()
    out
}

## Fetch at most n records from the opened resultSet (n = -1 means
## all records, n=0 means extract as many as "default_fetch_rec",
## as defined by PostgreSQLDriver (see describe(drv, T)).
## The returned object is a data.frame.
## Note: The method dbHasCompleted() on the resultSet tells you whether
## or not there are pending records to be fetched.
##
## TODO: Make sure we don't exhaust all the memory, or generate
## an object whose size exceeds option("object.size").  Also,
## are we sure we want to return a data.frame?
postgresqlFetch <- function(res, n=0, ...) {
    n <- as(n, "integer")
    rsId <- as(res, "integer")
    rel <- .Call(RS_PostgreSQL_fetch, rsId, nrec = n)
    ## create running row index as of previous fetch (if any)
    cnt <- dbGetRowCount(res)
    nrec <- length(rel[[1]])
    indx <- seq.int(from = cnt - nrec + 1, length.out = nrec)
    attr(rel, "row.names") <- indx
    if(usingR())
        class(rel) <- "data.frame"
    else
        oldClass(rel) <- "data.frame"

    drop_infinity <- function(x){ if(is.na(x)) NA else if(x == 'infinity' || x == '-infinity'){warning(x, ": infinity in date data detected") ; NA} else x}
    flds <- dbGetInfo(res)$fieldDescription[[1]]$type
    if(length(rel[,1])>0){
        for(i in 1:length(flds)) {
            if(flds[[i]] == 1114) {  ## 1114 corresponds to Timestamp without TZ (mapped to POSIXct class)
                rel[,i] <- sapply(rel[,i], drop_infinity) 
                rel[,i] <- as.POSIXct(rel[,i])
            } else if(flds[[i]] == 1082) {  ## 1082 corresponds to Date (mapped to Date class)
                rel[,i] <- sapply(rel[,i], drop_infinity) 
                rel[,i] <- as.Date(rel[,i])
            } else if(flds[[i]] == 1184)  {  ## 1184 corresponds to Timestamp with TimeZone
                rel[,i] <- sapply(rel[,i], drop_infinity) 
                rel[,i] <- as.POSIXct(sub('([+-]..)$', '\\100', sub(':(..)$','\\1' ,rel[,i])), format="%Y-%m-%d %H:%M:%OS%z")
            }
        }
    }
    rel
}

## Note that originally we had only resultSet both for SELECTs
## and INSERTS, ...  Later on we created a base class dbResult
## for non-Select SQL and a derived class resultSet for SELECTS.
postgresqlResultInfo <- function(obj, what = "", ...) {
    if(!isPostgresqlIdCurrent(obj))
        stop(paste("expired", class(obj), deparse(substitute(obj))))
    id <- as(obj, "integer")
    info <- .Call(RS_PostgreSQL_resultSetInfo, id)
    if(!missing(what))
        info[what]
    else
        info
}

postgresqlDescribeResult <- function(obj, verbose = FALSE, ...) {
    if(!isPostgresqlIdCurrent(obj)){
        print(obj)
        invisible(return(NULL))
    }
    print(obj)
    cat("  Statement:", dbGetStatement(obj), "\n")
    cat("  Has completed?", if(dbHasCompleted(obj)) "yes" else "no", "\n")
    cat("  Affected rows:", dbGetRowsAffected(obj), "\n")
    cat("  Rows fetched:", dbGetRowCount(obj), "\n")
    flds <- dbColumnInfo(obj)
    if(verbose && !is.null(flds)){
        cat("  Fields:\n")
        out <- print(dbColumnInfo(obj))
    }
    invisible(NULL)
}

postgresqlCloseResult <- function(res, ...) {
    if(!isPostgresqlIdCurrent(res))
        return(TRUE)
    rsId <- as(res, "integer")
    .Call(RS_PostgreSQL_closeResultSet, rsId)
}

## Use NULL, "", or 0 as row.names to prevent using any field as row.names.
postgresqlReadTable <- function(con, name, row.names = "row.names", check.names = TRUE, ...) {
    out <- dbGetQuery(con, paste("SELECT * from", postgresqlTableRef(name)))
    if(check.names)
        names(out) <- make.names(names(out), unique = TRUE)
    ## should we set the row.names of the output data.frame?
    nms <- names(out)
    j <- switch(mode(row.names),
                "character" = if(row.names=="") 0 else
                match(tolower(row.names), tolower(nms),
                      nomatch = if(missing(row.names)) 0 else -1),
                "numeric" = row.names,
                "NULL" = 0,
                0)
    if(j==0)
        return(out)
    if(j<0 || j>ncol(out)){
        warning("row.names not set on output data.frame (non-existing field)")
        return(out)
    }
    rnms <- as.character(out[,j])
    if(all(!duplicated(rnms))){
        out <- out[,-j, drop = FALSE]
        row.names(out) <- rnms
    } else warning("row.names not set on output (duplicate elements in field)")
    out
}

postgresqlImportFile <- function(con, name, value, field.types = NULL, overwrite = FALSE,
                                 append = FALSE, header, row.names, nrows = 50, sep = ",",
                                 eol="\n", skip = 0, quote = '"', ...) {
    if(overwrite && append)
        stop("overwrite and append cannot both be TRUE")

    new.con <- con

    if(dbExistsTable(con,name)){
        if(overwrite){
            if(!dbRemoveTable(con, name)){
                warning(paste("table", name, "couldn't be overwritten"))
                return(FALSE)
            }
        }
        else if(!append){
            warning(paste("table", name, "exists in database: aborting dbWriteTable"))
            return(FALSE)
        }
    }

    ## compute full path name (have R expand ~, etc)
    fn <- file.path(dirname(value), basename(value))
    if(missing(header) || missing(row.names)){
        f <- file(fn, open="r")
        if(skip>0)
            readLines(f, n=skip)
        txtcon <- textConnection(readLines(f, n=2))
        flds <- count.fields(txtcon, sep)
        close(txtcon)
        close(f)
        nf <- length(unique(flds))
    }
    if(missing(header)){
        header <- nf==2
    }
    if(missing(row.names)){
        if(header)
            row.names <- if(nf==2) TRUE else FALSE
        else
            row.names <- FALSE
    }

    new.table <- !dbExistsTable(con, name)
    if(new.table){
        ## need to init table, say, with the first nrows lines
        d <- read.table(fn, sep=sep, header=header, skip=skip, nrows=nrows, ...)
        sql <-
            postgresqlBuildTableDefinition(new.con, name, obj=d, field.types = field.types,
                                   row.names = row.names)
        rs <- try(dbSendQuery(new.con, sql))
        if(inherits(rs, ErrorClass)){
            warning("could not create table: aborting postgresqlImportFile")
            return(FALSE)
        }
        else
            dbClearResult(rs)
    }
    else if(!append){
        warning(sprintf("table %s already exists -- use append=TRUE?", name))
    }

    fmt <- paste("COPY %s FROM '%s' ","WITH DELIMITER AS '%s' ",
                 if(!is.null(quote)) "CSV  QUOTE AS  '%s'", sep="")

    if(is.null(quote))
        sql <- sprintf(fmt, name,fn, sep)
    else
        sql <- sprintf(fmt, name,fn, sep, quote)

    rs <- try(dbSendQuery(new.con, sql))
    if(inherits(rs, ErrorClass)){
        warning("could not load data into table")
        return(FALSE)
    }
    dbClearResult(rs)
    TRUE
}

## Create table "name" (must be an SQL identifier) and populate
## it with the values of the data.frame "value"
## TODO: This function should execute its sql as a single transaction,
##       and allow converter functions.
postgresqlWriteTable <- function(con, name, value, field.types, row.names = TRUE,
                                 overwrite = FALSE, append = FALSE, ..., allow.keywords = FALSE) {
    if(overwrite && append)
        stop("overwrite and append cannot both be TRUE")
    if(!is.data.frame(value))
        value <- as.data.frame(value)
    if(row.names){
        value <- cbind(row.names(value), value)  ## can't use row.names= here
        names(value)[1] <- "row.names"
    }
    if(missing(field.types) || is.null(field.types)){
        ## the following mapping should be coming from some kind of table
        ## also, need to use converter functions (for dates, etc.)
        field.types <- sapply(value, dbDataType, dbObj = con)
    }

    i <- match("row.names", names(field.types), nomatch=0)
    if(i>0) ## did we add a row.names value?  If so, it's a text field.
        ## MODIFIED -- Sameer
        field.types[i] <- dbDataType(dbObj=con, field.types[row.names])
    new.con <- con

    if(dbExistsTable(con,name)){
        if(overwrite){
            if(!dbRemoveTable(con, name)){
                warning(paste("table", name, "couldn't be overwritten"))
                return(FALSE)
            }
        }
        else if(!append){
            warning(paste("table",name,"exists in database: aborting assignTable"))
            return(FALSE)
        }
    }
    if(!dbExistsTable(con,name)){      ## need to re-test table for existance
        ## need to create a new (empty) table
        sql1 <- paste("create table ", postgresqlTableRef(name), "\n(\n\t", sep="")
        sql2 <- paste(paste(postgresqlQuoteId(names(field.types)), field.types), collapse=",\n\t",
                      sep="")
        sql3 <- "\n)\n"
        sql <- paste(sql1, sql2, sql3, sep="")
        rs <- try(dbSendQuery(new.con, sql))
        if(inherits(rs, ErrorClass)){
            warning("could not create table: aborting assignTable")
            return(FALSE)
        } else {
            dbClearResult(rs)
        }
    }

    ## convert columns we can't handle in C code
    value[] <- lapply(value, function(z) {
        if(is.object(z) && !is.factor(z)) as.character(z) else z
    })
    oldenc <- dbGetQuery(new.con, "SHOW client_encoding")
    postgresqlpqExec(new.con, "SET CLIENT_ENCODING TO 'UTF8'")
    sql4 <- paste("COPY", postgresqlTableRef(name),"(",paste(postgresqlQuoteId(names(value)),collapse=","),") FROM STDIN")
    postgresqlpqExec(new.con, sql4)
    postgresqlCopyInDataframe(new.con, value)
    rs<-postgresqlgetResult(new.con)

    retv <- TRUE
    if (inherits(rs, ErrorClass)) {
        warning("could not load data into table")
        retv <- FALSE
    }

    dbClearResult(rs)
    sql5 <- paste("SET CLIENT_ENCODING TO '", oldenc, "'", sep="")
    dbGetQuery(new.con, sql5)

    retv
}

postgresqlBuildTableDefinition <- function(dbObj, name, obj, field.types = NULL, row.names = TRUE, ...) {
    if(!is.data.frame(obj))
        obj <- as.data.frame(obj)
    if(!is.null(row.names) && row.names){
        obj  <- cbind(row.names(obj), obj)  ## can't use row.names= here
        names(obj)[1] <- "row.names"
    }
    if(is.null(field.types)){
        ## the following mapping should be coming from some kind of table
        ## also, need to use converter functions (for dates, etc.)
        field.types <- sapply(obj, dbDataType, dbObj = dbObj)
    }
    i <- match("row.names", names(field.types), nomatch=0)
    if(i>0) ## did we add a row.names value?  If so, it's a text field.
        field.types[i] <- dbDataType(dbObj, field.types$row.names)

    ## need to create a new (empty) table
    flds <- paste(postgresqlQuoteId(names(field.types)), field.types)
    paste("CREATE TABLE", postgresqlTableRef(name), "\n(", paste(flds, collapse=",\n\t"), "\n)")
}


## find a suitable SQL data type for the R/S object obj
## TODO: Lots and lots!! (this is a very rough first draft)
## need to register converters, abstract out PostgreSQL and generalize
## to Oracle, Informix, etc.  Perhaps this should be table-driven.
## NOTE: PostgreSQL data types differ from the SQL92 (e.g., varchar truncate
## trailing spaces).
postgresqlDataType <- function(obj, ...) {
    rs.class <- data.class(obj)
    if(rs.class=="numeric"){
        sql.type <- if(is(obj,"integer")) "integer" else  "float8"
    }
    else {
        sql.type <- switch(rs.class,
                           character = "text",
                           logical = "bool",
                           factor = "text",
                           ordered = "text",
                           Date = "date",
                           POSIXct = "timestamp with time zone",
                           "text")
    }
    sql.type
}

postgresqlQuoteId <- function(identifiers){
    ret <- paste('"', gsub('"','""',identifiers), '"', sep="")
    ret
}
postgresqlTableRef <- function(identifiers){
    ret <- paste('"', gsub('"','""',identifiers), '"', sep="", collapse=".")
    ret
}

## the following reserved words were taken from ["RESERVED" of postgres colomn in ] Table C.1 in Appendix C
## of the PostgreSQL Manual 8.3.1.
.PostgreSQLKeywords <- c("ALL", "ANALYSE", "ANALYZE", "AND", "ANY",
                         "ARRAY", "AS", "ASC", "ASYMMETRIC", "AUTHORIZATION", "BETWEEN",
                         "BINARY", "BOTH", "CASE", "CAST", "CHECK", "COLLATE", "COLUMN",
                         "CONSTRAINT", "CREATE", "CROSS", "CURRENT_DATE", "CURRENT_ROLE",
                         "CURRENT_TIME", "CURRENT_TIMESTAMP", "CURRENT_USER", "DEFAULT",
                         "DEFERRABLE", "DESC", "DISTINCT", "DO", "ELSE", "END", "EXCEPT",
                         "FALSE", "FOR", "FOREIGN", "FREEZE", "FROM", "FULL", "GRANT", "GROUP",
                         "HAVING", "ILIKE", "IN", "INITIALLY", "INNER","INTERSECT", "INTO",
                         "IS", "ISNULL", "JOIN", "LEADING", "LEFT", "LIKE", "LIMIT",
                         "LOCALTIME", "LOCALTIMESTAMP", "NATURAL", "NEW", "NOT", "NULL", "OFF",
                         "OFFSET", "OLD", "ON", "ONLY", "OR", "ORDER", "OUTER", "OVERLAPS",
                         "PLACING", "PRIMARY", "REFERENCES", "RESERVED", "SELECT",
                         "SESSION_USER", "SIMILAR", "SOME", "SYMMETRIC", "TABLE", "THEN", "TO",
                         "TRAILING", "TRUE", "UNION", "UNIQUE", "USER", "USING", "VERBOSE",
                         "WHEN", "WHERE", "WITH" )

## Test of data types, based on earlier version in inst/devTests
##
## Dirk Eddelbuettel, 21 Oct 2008

if ((Sys.getenv("POSTGRES_USER") != "") &
    (Sys.getenv("POSTGRES_HOST") != "") &
    (Sys.getenv("POSTGRES_DATABASE") != "")) {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")
    ## can't print result as it contains process id which changes  print(summary(drv))

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))

    if (dbExistsTable(con, "tempostgrestable"))
        dbRemoveTable(con, "tempostgrestable")

    ## Test the numeric mapping
    dbGetQuery(con, "create table tempostgrestable (intcolumn integer, floatcolumn float);")

    i <- as.integer(10)
    j <- as.numeric(56.6)

    sql <- paste("insert into tempostgrestable ",
                 "values (",i, "," ,j ,") ", sep="")
    res <- dbGetQuery(con, sql)


    dat <- dbReadTable(con, "tempostgrestable")
    dbRemoveTable(con, "tempostgrestable")
    cat("Read Numeric values\n")

    ## now test the types of the colums we got
    stopifnot( class(dat[,1]) == "integer" )
    stopifnot( class(dat[,2]) == "numeric" )
    cat("GOOD -- all numeric types are as expected\n")

    ## and test the values
    stopifnot( identical( dat[1,1], i))
    stopifnot( identical( dat[1,2], j))
    cat("GOOD -- all numeric values are as expected\n")

    ## Test the logical mapping
    if (dbExistsTable(con, "testlogical"))
        dbRemoveTable(con, "testlogical")
    dbGetQuery(con,"create table testlogical (col1 boolean, col2 boolean)")

    i <- as.logical(TRUE)
    j <- as.logical(FALSE)

    sql <- paste("insert into testlogical ",
                 "values (",i, "," ,j ,") ", sep="")
    res <- dbGetQuery(con, sql);

    dat <- dbReadTable(con, "testlogical")
    dbRemoveTable(con, "testlogical")
    cat("Read Logical values\n")

    ## now test the types of the colums we got
    stopifnot( class(dat[,1]) == "logical" )
    stopifnot( class(dat[,2]) == "logical" )
    cat("GOOD -- all logical types are as expected\n")

    ## and test the values
    stopifnot( identical( dat[1,1], i))
    stopifnot( identical( dat[1,2], j))
    cat("GOOD -- all logical values are as expected\n")

    ## Test the character mapping
    if (dbExistsTable(con, "testchar"))
        dbRemoveTable(con, "testchar")
    dbGetQuery(con,"create table testchar (code char(3),city varchar(20),country text);")

    i <- as.character("IN")
    j <- as.character("Hyderabad")
    k <- as.character("India")

    sql <- paste("insert into testchar ",
                 "values ('",i,"' , '",j ,"' , '",k,"') ", sep="")
    res <- dbGetQuery(con, sql);

    dat <- dbReadTable(con, "testchar")
    cat("Read Character values\n")

    ## now test the types of the colums we got
    stopifnot( class(dat[,1]) == "character" )
    stopifnot( class(dat[,2]) == "character" )
    stopifnot( class(dat[,3]) == "character" )
    cat("GOOD -- all character types are as expected\n")

    ## and test the values
    ##stopifnot( identical( dat[1,1], i))
    stopifnot( identical( dat[1,2], j))
    stopifnot( identical( dat[1,3], k))
    cat("GOOD -- all character values are as expected\n")

    dbRemoveTable(con, "testchar")
    dbRemoveTable(con, "tempostgrestable")

    dbDisconnect(con)
    dbUnloadDriver(drv)

    cat("DONE\n")
}

## connectWithNull test
##
## test for the 'Issue 2' on the Google Code issue log
## reported in April 2009, still open ?
##
## Assumes that
##  a) PostgreSQL is running, and
##  b) the current user can connect
## both of which are not viable for release but suitable while we test
##
## Dirk Eddelbuettel, 03 Oct 2009

## only run this if this env.var is set correctly
if (Sys.getenv("POSTGRES_USER") != "" & Sys.getenv("POSTGRES_HOST") != "" & Sys.getenv("POSTGRES_DATABASE") != "") {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))
    stopifnot(require(datasets))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")

    ## connect to the default db -- replacing any of these with NULL will lead to
    ## a stop() call and a return to the R prompt rather than a segfault
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))
    
    st <- (postgresqlEscapeStrings(con,"aaa"))
    print(st)
    st2 <- (postgresqlEscapeStrings(con,"aa'a"))
    print(st2)

    ## and disconnect
    dbDisconnect(con)
}

## dbExistsq test with schema name: reported as Issue 28 at
## http://code.google.com/p/rpostgresql/issues/detail?id=28
##
## Assumes that
##  a) PostgreSQL is running, and
##  b) the current user can connect
## both of which are not viable for release but suitable while we test
##
## Dirk Eddelbuettel, 10 Sep 2009

## only run this if this env.var is set correctly
if (Sys.getenv("POSTGRES_USER") != "" & Sys.getenv("POSTGRES_HOST") != "" & Sys.getenv("POSTGRES_DATABASE") != "") {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))
    stopifnot(require(datasets))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))

    if (dbExistsTable(con, "rock.data")) {
        cat("Removing rock'data\n")
        dbRemoveTable(con, "rock.data")
    }


    dbWriteTable(con, "rock.data", rock)
    ## run a simple dbExists
    cat("Does rock.data exist? \n")
    res <- dbExistsTable(con, "rock.data")
    if(res){
      cat("PASS: true\n")
    }else{
      cat("FAIL: false\n")
    }

    ## 
    cat("create schema testschema and change the search_path\n")

    dbGetQuery(con, 'CREATE SCHEMA testschema')
    dbGetQuery(con, 'SET search_path TO testschema')
    cat("Does rock.data exist? \n")
    res <- dbExistsTable(con, "rock.data")
    if(res){
      cat("FAIL: true despite search_path change\n")
    }else{
      cat("PASS: false as the search_path changed\n")
    }

    cat("Does testschema.\"rock.data\" exist? \n")
    res <- dbExistsTable(con, c('testschema', "rock.data"))
    if(res){
      cat("FAIL: true despite testschema specified\n")
    }else{
      cat("PASS: false as the testschema specified\n")
    }

    cat("Does public.\"rock.data\" exist? \n")
    res <- dbExistsTable(con, c('public', "rock.data"))
    if(res){
      cat("PASS: true despite search_path change\n")
    }else{
      cat("FAIL: false as the search_path changed\n")
    }

    
    cat("write in current schema\n")
    dbWriteTable(con, "rock.data", rock)
    cat("Does rock.data exist? \n")
    res <- dbExistsTable(con, "rock.data")
    if(res){
      cat("PASS: true\n")
    }else{
      cat("FAIL: false\n")
    }

    ## cleanup
    dbGetQuery(con, 'DROP TABLE "public"."rock.data"')
    dbGetQuery(con, 'DROP TABLE "testschema"."rock.data"')
    dbGetQuery(con, 'DROP schema "testschema"')

    ## and disconnect
    dbDisconnect(con)
}

## dbWriteTable test
##
## Assumes that
##  a) PostgreSQL is running, and
##  b) the current user can connect
## both of which are not viable for release but suitable while we test
##
## Dirk Eddelbuettel, 10 Sep 2009

## only run this if this env.var is set correctly
if (Sys.getenv("POSTGRES_USER") != "" & Sys.getenv("POSTGRES_HOST") != "" & Sys.getenv("POSTGRES_DATABASE") != "") {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))


    if (dbExistsTable(con, "rockdata")) {
        print("Removing rockdata\n")
        dbRemoveTable(con, "rockdata")
    }

    dbGetQuery(con, "set client_encoding to 'UTF-8'")
    difficultstrings <- c("normal", "t\tab", "v\vertical tab", "n\newline", "r carriage \retern", "back \\ slash", "f\form feed")
    df <- data.frame(strings=difficultstrings)

    dbWriteTable(con, "rockdata", df)

    ## run a simple query and show the query result
    res <- dbGetQuery(con, "select * from rockdata")
    print(res)
    print("Removing rockdata\n")
    dbRemoveTable(con, "rockdata")

    difficultstringe <- c("normal", "m\u00fc\u00df")
    df <- data.frame(strings=difficultstringe)
    tryres <- try({dbWriteTable(con, "rockdata", df)
        res <- dbGetQuery(con, "select * from rockdata")
        for(n in 1:2){
            cat(paste(as.character(n), "\t"))
            cat(res[n,2])
            cat("\n")
        }
        print("Removing rockdata\n")
        dbRemoveTable(con, "rockdata")
    })
    if(tryres != TRUE){
       cat("FAIL:  could not write small umlaut u and ligature sz.\n")
       cat("       This might be no problem for you if you don't use those special characters.\n")
       cat("       Otherwise, please check for the server encoding.\n")
       cat("       Database encoding is usually set at the time of createdb.\n")
       cat("       You can see for more information on how to setup at \n")
       cat("       http://www.postgresql.org/docs/9.1/static/multibyte.html\n\n")
    }else{
       cat("PASS:  could write small umlaut u and ligature sz\n")
    }
   
    difficultstringk <- c("normal", "kanji\u6f22\u5b57")
    df <- data.frame(strings=difficultstringk)
    tryres <- try({dbWriteTable(con, "rockdata", df)
        res <- dbGetQuery(con, "select * from rockdata")
        for(n in 1:2){
            cat(paste(as.character(n), "\t"))
            cat(res[n,2])
            cat("\n")
        }
        print("Removing rockdata\n")
        dbRemoveTable(con, "rockdata")
    })
    if(tryres != TRUE){
       cat("FAIL:  could not write kanji.\n")
       cat("       This might be no problem for you if you don't use multibyte characters.\n")
       cat("       Otherwise, please check for the server encoding.\n")
       cat("       Database encoding is usually set at the time of createdb.\n")
       cat("       You can see for more information on how to setup at \n")
       cat("       http://www.postgresql.org/docs/9.1/static/multibyte.html\n\n")
    }else{
       cat("PASS:  could write kanji\n")
    }

    ## cleanup
    if (dbExistsTable(con, "rockdata")) {
        print("Removing rockdata\n")
        dbRemoveTable(con, "rockdata")
    }

    if (dbExistsTable(con, "tempostgrestable"))
        dbRemoveTable(con, "tempostgrestable")

    ## Test the numeric mapping
    dbGetQuery(con, "create table tempostgrestable (intcolumn integer, floatcolumn float);")

    i <- as.integer(10)
    j <- as.numeric(56.6)

    sql <- paste("insert into tempostgrestable ",
                 "values (",i, "," ,j ,") ", sep="")
    res <- dbGetQuery(con, sql)

    dat <- dbReadTable(con, "tempostgrestable")
    dbRemoveTable(con, "tempostgrestable")
    res <- dbWriteTable(con, "numerictable", dat)
    dat <- dbReadTable(con, "numerictable")
    dbRemoveTable(con, "numerictable")
    cat("Read Numeric values\n")

    ## now test the types of the colums we got
    if( class(dat[,1]) == "integer" ) {
       cat("PASS -- all integer is as expected\n")
    }else{
       cat(paste("FAIL -- an integer became ", class(dat[,1]), "\n"))
    }
    stopifnot( class(dat[,2]) == "numeric" )

    ## and test the values
    if( identical( dat[1,1], i)){
       cat("PASS integer value is preserved")
    }else{
       cat(paste("FAIL:", i, "changed to", dat[1,1], "\n")) 
    }
    stopifnot( identical( dat[1,2], j))
    cat("GOOD -- all numeric values are as expected\n")

    ## Test the logical mapping
    if (dbExistsTable(con, "testlogical"))
        dbRemoveTable(con, "testlogical")
    dbGetQuery(con,"create table testlogical (col1 boolean, col2 boolean)")

    i <- as.logical(TRUE)
    j <- as.logical(FALSE)

    sql <- paste("insert into testlogical ",
                 "values (",i, "," ,j ,") ", sep="")
    res <- dbGetQuery(con, sql);

    dat <- dbReadTable(con, "testlogical")
    res <- dbWriteTable(con, "logicaltable", dat)
    dbRemoveTable(con, "testlogical")
    dat2 <- dbReadTable(con,"logicaltable")
    dbRemoveTable(con, "logicaltable")
    cat("Read Logical values\n")

    ## now test the types of the colums we got
    stopifnot( class(dat2[,1]) == "logical" )
    stopifnot( class(dat2[,2]) == "logical" )
    cat("GOOD -- all logical types are as expected\n")

    ## and test the values
    stopifnot( identical( dat2[1,1], i))
    stopifnot( identical( dat2[1,2], j))
    cat("GOOD -- all logical values are as expected\n")

    ## Test the character mapping
    if (dbExistsTable(con, "testchar"))
        dbRemoveTable(con, "testchar")
    dbGetQuery(con,"create table testchar (code char(3),city varchar(20),country text);")

    i <- as.character("IN")
    j <- as.character("Hyderabad")
    k <- as.character("India")

    sql <- paste("insert into testchar ",
                 "values ('",i,"' , '",j ,"' , '",k,"') ", sep="")
    res <- dbGetQuery(con, sql);

    dat <- dbReadTable(con, "testchar")
    dbRemoveTable(con, "testchar")
    dbWriteTable(con, "testchar", dat)
    dat <- dbReadTable(con, "testchar")
    cat("Read Character values\n")

    ## now test the types of the colums we got
    stopifnot( class(dat[,1]) == "character" )
    stopifnot( class(dat[,2]) == "character" )
    stopifnot( class(dat[,3]) == "character" )
    cat("GOOD -- all character types are as expected\n")

    ## and test the values
    ##stopifnot( identical( dat[1,1], i))
    stopifnot( identical( dat[1,2], j))
    stopifnot( identical( dat[1,3], k))
    cat("GOOD -- all character values are as expected\n")

    dbRemoveTable(con, "testchar")
    dbRemoveTable(con, "tempostgrestable")

    ## Test the numeric mapping
    dbGetQuery(con, "create table tempostgrestable (intcolumn date, floatcolumn timestamp with time zone);")

    sql <- paste("insert into tempostgrestable ",
                 "values ('2011-03-07', '2011-03-07 16:30:39') ")
    res <- dbGetQuery(con, sql)

    dat <- dbReadTable(con, "tempostgrestable")
    dbRemoveTable(con, "tempostgrestable")
    dbWriteTable(con, "tempostgrestable2", dat)
    dat2 <- dbReadTable(con, "tempostgrestable2")
    dbRemoveTable(con, "tempostgrestable2")
    cat("Check that read after write gets the same data types\n")

    ## now test the types of the colums we got
    if( class(dat2[1,1]) == "Date" ){
        cat("PASS -- Date type is as expected\n")
    }else{
        cat("FAIL -- Date type is other than Date: ")
        cat(class(dat2[1,1]))
        cat("\n")
    }
    if( class(dat2[1,2])[1] == "POSIXct" ){
        cat("PASS -- TIMESTAMP is received as POSIXct\n")
    }else{
        cat("FAIL -- TIMESTAMP is other than POSIXct: ")
        cat(class(dat2[1,2]))
        cat("\n")
    }


    dbDisconnect(con)
    dbUnloadDriver(drv)

    cat("DONE\n")
}

## First rough version of a test script
##
## Assumes that
##  a) PostgreSQL is running, and
##  b) the current user can connect
## both of which are not viable for release but suitable while we test
##
## Dirk Eddelbuettel, 02 Jul 2008
##                    21 Oct 2008   make conditional on environment variables

## only run this if this env.var is set correctly
if (Sys.getenv("POSTGRES_USER") != "" & Sys.getenv("POSTGRES_HOST") != "" & Sys.getenv("POSTGRES_DATABASE") != "") {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")
    ## can't print result as it contains process id which changes  print(summary(drv))

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))

    ## run a simple query and show the query result
    res <- dbGetQuery(con, paste("select datname,encoding,datallowconn from pg_database",
                                 "where datname like 'template%' order by datname"))
    print(res)

    ## and disconnect
    dbDisconnect(con)
}

## Test of data types, based on earlier version in inst/devTests
##
## Dirk Eddelbuettel, 21 Oct 2008

if ((Sys.getenv("POSTGRES_USER") != "") &
    (Sys.getenv("POSTGRES_HOST") != "") &
    (Sys.getenv("POSTGRES_DATABASE") != "")) {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")
    ## can't print result as it contains process id which changes  print(summary(drv))

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))

    if (dbExistsTable(con, "tempostgrestable"))
        dbRemoveTable(con, "tempostgrestable")

    ## Test the numeric mapping
    dbGetQuery(con, "create table tempostgrestable (intcolumn date, floatcolumn timestamp with time zone);")

    sql <- paste("insert into tempostgrestable ",
                 "values ('2011-03-07', '2011-03-07 16:30:39') ")
    res <- dbGetQuery(con, sql)

    dat <- dbReadTable(con, "tempostgrestable")
    dbRemoveTable(con, "tempostgrestable")
    cat("Read Date and TIMESTAMP values\n")

    ## now test the types of the colums we got
    if( class(dat[1,1]) == "Date" ){
        cat("PASS -- Date type is as expected\n")
    }else{
        cat("FAIL -- Date type is other than Date: ")
        cat(class(dat[1,1]))
        cat("\n")
    }
    if( class(dat[1,2])[1] == "POSIXct" ){
        cat("PASS -- TIMESTAMP is received as POSIXct\n")
    }else{
        cat("FAIL -- TIMESTAMP is other than POSIXct: ")
        cat(class(dat[1,2]))
        cat("\n")
    }

    dbWriteTable(con, "tempostgrestable2", dat)
    dat2 <- dbReadTable(con, "tempostgrestable2")
    dbRemoveTable(con, "tempostgrestable2")
    cat("Check that read after write gets the same data types\n")

    ## now test the types of the colums we got
    if( class(dat2[1,1]) == "Date" ){
        cat("PASS -- Date type is as expected\n")
    }else{
        cat("FAIL -- Date type is other than Date: ")
        cat(class(dat2[1,1]))
        cat("\n")
    }
    if( class(dat2[1,2])[1] == "POSIXct" ){
        cat("PASS -- TIMESTAMP is received as POSIXct\n")
    }else{
        cat("FAIL -- TIMESTAMP is other than POSIXct: ")
        cat(class(dat2[1,2]))
        cat("\n")
    }


    dbDisconnect(con)
    dbUnloadDriver(drv)

    cat("DONE\n")
}

## createTableMixedCaseTest test
##
##
## Assumes that
##  a) PostgreSQL is running, and
##  b) the current user can connect
## both of which are not viable for release but suitable while we test
##
## Neil Tiffin, 30 Oct 2009

## only run this if this env.var is set correctly
if (Sys.getenv("POSTGRES_USER") != "" & Sys.getenv("POSTGRES_HOST") != "" & Sys.getenv("POSTGRES_DATABASE") != "") {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=Sys.getenv("POSTGRES_PORT"))



    res <- dbGetQuery(con, "create table Foo1 (f1 int)")
    res <- dbGetQuery(con, "create table \"Foo2\" (f1 int)")

    cat("Test should create foo1 and Foo2 tables\n")
    ## res <- dbGetQuery(con, "SELECT * FROM information_schema.tables WHERE table_schema = 'public'")
    ## print res

    if (dbExistsTable(con, "Foo1")) {
        cat("FAIL - Foo1 Table exists.\n")
    }
    else {
        cat("Pass - Foo1 Table does not exist.\n")
    }

    if (dbExistsTable(con, "foo1")) {
        cat("Pass - foo1 Table exists.\n")
    }
    else {
        cat("FAIL - foo1 Table does not exist.\n")
    }

    if (dbExistsTable(con, "Foo2")) {
        cat("Pass - Foo2 Table exists.\n")
    }
    else {
        cat("FAIL - Foo2 Table does not exist.\n")
    }

    if (dbExistsTable(con, "foo2")) {
        cat("FAIL - foo2 Table exists.\n")
    }
    else {
        cat("Pass - foo2 Table does not exist.\n")
    }

    if (dbExistsTable(con, "\"Foo2\"")) {
        cat("FAIL - \"Foo2\" Table exists.\n")
    }
    else {
        cat("Pass - \"Foo2\" Table does not exist.\n")
    }

    if (dbExistsTable(con, "\"foo2\"")) {
        cat("FAIL - \"foo2\" Table exists.\n")
    }
    else {
        cat("Pass - \"foo2\" Table does not exist.\n")
    }

    res <- dbGetQuery(con, "drop table Foo1")
    res <- dbGetQuery(con, "drop table \"Foo2\"")
    ## and disconnect
    dbDisconnect(con)
}

## Class: dbObjectId
## $Id$

## This package was developed as a part of Summer of Code program organized by Google.
## Thanks to David A. James & Saikat DebRoy, the authors of RMySQL package.
## Code from RMySQL package was reused with the permission from the authors.
## Also Thanks to my GSoC mentor Dirk Eddelbuettel for helping me in the development.


##
## This mixin helper class is NOT part of the database interface definition,
## but it is extended by the Oracle, MySQL,PostgreSQL and SQLite implementations
## to allow us to conviniently (and portably)
## implement all database foreign objects methods (i.e., methods for show(),
## print() format() the dbManger, dbConnection, dbResultSet, etc.)
## A dbObjectId is an  identifier into an actual remote database objects.
## This class and its derived classes <driver-manager>Object need to
## be VIRTUAL to avoid coercion (green book, p.293) during method dispatching.

setClass("dbObjectId", representation(Id = "integer", "VIRTUAL"))

## coercion methods
setAs("dbObjectId", "integer",
      def = function(from) as(slot(from,"Id"), "integer")
      )
setAs("dbObjectId", "numeric",
      def = function(from) as(slot(from, "Id"), "integer")
      )
setAs("dbObjectId", "character",
      def = function(from) as(slot(from, "Id"), "character")
      )

## formating, showing, printing,...
setMethod("format", "dbObjectId",
          def = function(x, ...) {
              paste("(", paste(as(x, "integer"), collapse=","), ")", sep="")
          },
          valueClass = "character"
          )

setMethod("show", "dbObjectId", def = function(object) print(object))

setMethod("print", "dbObjectId",
          def = function(x, ...){
              expired <- if(isPostgresqlIdCurrent(x)) "" else "Expired "
              str <- paste("<", expired, class(x), ":", format(x), ">", sep="")
              cat(str, "\n")
              invisible(NULL)
          }
          )

## verify that obj refers to a currently open/loaded database
isPostgresqlIdCurrent <- function(obj) {
    obj <- as(obj, "integer")
    .Call(RS_DBI_validHandle, obj)
}

## Test of bytea conversion with insert and retrieve to the db.
##

if ((Sys.getenv("POSTGRES_USER") != "") &
    (Sys.getenv("POSTGRES_HOST") != "") &
    (Sys.getenv("POSTGRES_DATABASE") != "")) {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))

    if (dbExistsTable(con, "texttable"))
        dbRemoveTable(con, "texttable")

        x_df <- data.frame(x = c("a", "ö", "漢字"), stringsAsFactors = FALSE)

        dbGetQuery(con, "SHOW server_encoding")
        dbGetQuery(con, "SET client_encoding = 'UTF8'")
        dbGetQuery(con, "SHOW client_encoding")
# write with client_encoding = 'UTF8' and read back what was wrote
        dbWriteTable(con, "texttable", x_df, overwrite = TRUE)
        (x_db <- dbReadTable(con, "texttable"))
        (e <- Encoding(x_db$x))
        stopifnot( e[1] == "unknown" )
        stopifnot( e[2] == "UTF-8" )
        stopifnot( e[3] == "UTF-8" )
        cat("GOOD -- all encoding are as expected\n")

        x_df <- data.frame(x = c("a", "ö"), stringsAsFactors = FALSE)
        dbWriteTable(con, "texttable", x_df, overwrite = TRUE)
        dbGetQuery(con, "SET client_encoding = 'LATIN1'")
        dbGetQuery(con, "SHOW client_encoding")
# read with client_encoding = 'LATIN1' what was already in the database
        (x_db <- dbReadTable(con, "texttable"))
        (e <- Encoding(x_db$x))
        stopifnot( e[1] == "unknown" )
        stopifnot( e[2] == "latin1" )
        cat("GOOD -- all encoding are as expected\n")

# write with client_encoding = 'LATIN1' and read back what was wrote
        dbWriteTable(con, "texttable", x_df, overwrite = TRUE)
        (x_db <- dbReadTable(con, "texttable"))
        (e <- Encoding(x_db$x))
        stopifnot( e[1] == "unknown" )
        stopifnot( e[2] == "latin1" )
        cat("GOOD -- all encoding are as expected\n")

    dbRemoveTable(con, "texttable")
    dbDisconnect(con)
    dbUnloadDriver(drv)
    cat("DONE\n")
}else{
    cat("Skip because envirinmental variables are not set to tell the connection params.\n")
}

## Test of date and datetime types, based on earlier version in inst/devTests
##
## Dirk Eddelbuettel, 21 Oct 2008

dbTypeTests <- function(con, dateclass="timestamp without time zone") {
    cat("\n\n**** Trying with ", dateclass, "\n")

    if (dbExistsTable(con, "tempostgrestable"))
        dbRemoveTable(con, "tempostgrestable")

    dbGetQuery(con, paste("create table tempostgrestable (tt ", dateclass, ", zz integer);", sep=""))
    dbGetQuery(con, "insert into tempostgrestable values('2008-07-01 14:15:16.123', 1);")

    now <- ISOdatetime(2000,1,2,3,4,5.678)
    dbGetQuery(con, paste("insert into tempostgrestable values('", format(now), "', 2);", sep=""))

    res <- dbReadTable(con, "tempostgrestable")
    print(res)

    res <- dbSendQuery(con, "select tt from tempostgrestable;")
    data <- fetch(res, n=-1)
    print(dbColumnInfo(res))

    times <- data[,1]
    print(times)
    print(class(times[1]))

    print(diff(times))

    dbRemoveTable(con, "tempostgrestable")
    invisible(NULL)
}

## only run this if this env.var is set correctly
if ((Sys.getenv("POSTGRES_USER") != "") &
    (Sys.getenv("POSTGRES_HOST") != "") &
    (Sys.getenv("POSTGRES_DATABASE") != "")) {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))

    ## Force a timezone to make the tests comparable at different locations
    Sys.setenv("PGDATESTYLE"="German")
    Sys.setenv("TZ"="UTC")

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")
    ## can't print result as it contains process id which changes  print(summary(drv))

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))

    dbTypeTests(con, "timestamp")
#    dbTypeTests(con, "timestamp with time zone")
# output differs depending on the compute environment
    dbTypeTests(con, "date")

    dbDisconnect(con)
}

## Test of date and datetime types, based on earlier version in inst/devTests
##
## Dirk Eddelbuettel, 21 Oct 2008

dbTypeTests <- function(con, dateclass="timestamp without time zone", tz="UTC") {
    cat("\n\n**** Trying with ", dateclass, "\n")

    if (dbExistsTable(con, "tempostgrestable"))
        dbRemoveTable(con, "tempostgrestable")

    dbGetQuery(con, paste("create table tempostgrestable (tt ", dateclass, ", zz integer);", sep=""))
    insstring <- '2008-07-01 14:15:16.123+00'
    cat(paste('inserted string 1:', insstring))
    cat("\n")
    dbGetQuery(con, paste("insert into tempostgrestable values('", insstring, "', 1);", sep=""))

#    now <- ISOdatetime(2000,1,2,3,4,5.678)
    insstring <- '2000-01-02 03:04:05.678+00'
    cat(paste('inserted string 2:', insstring))
    cat("\n")
    dbGetQuery(con, paste("insert into tempostgrestable values('", insstring, "', 2);", sep=""))
    tt<-as.POSIXct(c("2008-07-01 23:45:16.123+0930","2000-01-02 13:34:05.678+1030"), format="%Y-%m-%d %H:%M:%OS%z")
    print(tt)

    dbGetQuery(con, paste("SET TIMEZONE TO '", tz, "'", sep=""))

    data <- dbGetQuery(con, "select tt from tempostgrestable;")

    times <- data[,1]
    print(times)
    if(as.double(tt[1]) == as.double(times[1])){
      cat('PASS\n')
    }else{
      cat('FAIL\n')
    }
    if(as.double(tt[2]) == as.double(times[2])){
      cat('PASS\n')
    }else{
      cat('FAIL\n')
    }


    dbRemoveTable(con, "tempostgrestable")
    invisible(NULL)
}

## only run this if this env.var is set correctly
if ((Sys.getenv("POSTGRES_USER") != "") &
    (Sys.getenv("POSTGRES_HOST") != "") &
    (Sys.getenv("POSTGRES_DATABASE") != "")) {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))

    ## Force a timezone to make the tests comparable at different locations
    Sys.setenv("TZ"="UTC")
    tt<-as.POSIXct(c("2008-07-01 23:45:16.123+0930","2000-01-02 13:34:05.678+1030"), format="%Y-%m-%d %H:%M:%OS%z")
    print(tt)

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")
    ## can't print result as it contains process id which changes  print(summary(drv))

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))

    cat('testing UTC')
    dbTypeTests(con, "timestamp")
    dbTypeTests(con, "timestamp with time zone")
    cat('testing Asia/Tokyo')
    dbTypeTests(con, "timestamp", tz="Asia/Tokyo")
    dbTypeTests(con, "timestamp with time zone", tz="Asia/Tokyo")
    cat('testing Australlia/South')
    dbTypeTests(con, "timestamp", tz="Australia/South")
    dbTypeTests(con, "timestamp with time zone", tz="Australia/South")
    cat('testing America/New_York')
    dbTypeTests(con, "timestamp", tz="America/New_York")
    dbTypeTests(con, "timestamp with time zone", tz="America/New_York")

    dbDisconnect(con)
}

## dbWriteTable test
##
## Assumes that
##  a) PostgreSQL is running, and
##  b) the current user can connect
## both of which are not viable for release but suitable while we test
##
## Dirk Eddelbuettel, 10 Sep 2009

## only run this if this env.var is set correctly
if (Sys.getenv("POSTGRES_USER") != "" & Sys.getenv("POSTGRES_HOST") != "" & Sys.getenv("POSTGRES_DATABASE") != "") {

    ## try to load our module and abort if this fails
    stopifnot(require(RPostgreSQL))
    stopifnot(require(datasets))

    ## load the PostgresSQL driver
    drv <- dbDriver("PostgreSQL")

    ## connect to the default db
    con <- dbConnect(drv,
                     user=Sys.getenv("POSTGRES_USER"),
                     password=Sys.getenv("POSTGRES_PASSWD"),
                     host=Sys.getenv("POSTGRES_HOST"),
                     dbname=Sys.getenv("POSTGRES_DATABASE"),
                     port=ifelse((p<-Sys.getenv("POSTGRES_PORT"))!="", p, 5432))


    if (dbExistsTable(con, "rockdata")) {
        print("Removing rockdata\n")
        dbRemoveTable(con, "rockdata")
    }
    cat("create incompatible table rockdata\n")
    dbGetQuery(con, "CREATE TABLE rockdata (a int, b varchar(5))")

    cat("write table to rockdata with append=TRUE\n")
    try({res <-dbWriteTable(con, 'rockdata', rock, append=TRUE, overwrite=FALSE)
        print(res)
        if(res == FALSE){
           cat("PASS as the return value is false\n")
        }else{
           cat("FAIL as the return value is true\n")
        }
    })

    cat("write table to rockdata\n")
    try({res <- dbWriteTable(con, 'rockdata', rock)
        if(res == FALSE){
        print(res)
           cat("PASS as the return value is false\n")
        }else{
           cat("FAIL as the return value is true\n")
        }
    })
  
    ## run a simple query and show the query result
    res <- dbGetQuery(con, "select * from rockdata limit 10")
    print(res)


    ## cleanup
    if (dbExistsTable(con, "rockdata")) {
        print("Removing rockdata\n")
        dbRemoveTable(con, "rockdata")
    }

    ## and disconnect
    dbDisconnect(con)
}

#!/usr/local/bin/bash

set -e

dbname="pgtemp"

createdb ${dbname} 
echo "Created ${dbname}"

cat <<EOF | R --slave
  library(RPostgreSQL)
  data(trees)
  drv <- dbDriver("PostgreSQL")
  con <- dbConnect(drv, dbname="${dbname}")
  res <- dbWriteTable(con, "trees", trees)
  dbDisconnect(con)
EOF

psql ${dbname} -c "select * from trees;" 

dropdb ${dbname}
echo "Deleted ${dbname}"

#!/bin/bash

set -e

dbname="pgtemp"

createdb ${dbname} 
echo "Created ${dbname}"

cat <<EOF | R --slave
  library(RPostgreSQL)
  data(trees)
  drv <- dbDriver("PostgreSQL")
  con <- dbConnect(drv, dbname="${dbname}")
  res <- dbWriteTable(con, "trees", trees)
  dbDisconnect(con)
EOF

psql ${dbname} -c "select * from trees;" 

dropdb ${dbname}
echo "Deleted ${dbname}"

%% TODO: Should we move this to the DBI package?
\name{postgresqlBuildTableDefinition}
\alias{postgresqlBuildTableDefinition}
\title{Build the SQL CREATE TABLE definition as a string}
\description{
   Build the SQL CREATE TABLE definition as a string for the
   input data.frame
}
\usage{
postgresqlBuildTableDefinition(dbObj, name, obj, 
	field.types = NULL, row.names = TRUE, ...)
}
\arguments{
  \item{dbObj}{any DBI object (used only to dispatch according to the 
    engine (e.g., MySQL, Oracle, PostgreSQL, SQLite) }
  \item{name}{name of the new SQL table}
  \item{obj}{an R object coerceable to data.frame for which we want 
  to create a table}
  \item{field.types}{optional named list of the types for each field 
    in \code{obj}} 
  \item{row.names}{logical, should row.name of \code{value} be exported
    as a \code{row\_names} field? Default is TRUE}
  \item{\dots}{reserved for future use}
}
\details{
  The output SQL statement is a simple \code{CREATE TABLE} with
  suitable for \code{dbGetQuery}
}
\value{
  An SQL string
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
  \code{\link{PostgreSQL}},
  \code{\link[DBI]{dbConnect}},
  \code{\link[DBI]{dbSendQuery}},
  \code{\link[DBI]{dbGetQuery}},
  \code{\link[DBI]{fetch}},
  \code{\link[DBI]{dbCommit}},
  \code{\link[DBI]{dbGetInfo}},
  \code{\link[DBI]{dbReadTable}}.
}
\keyword{methods}
\keyword{interface}
\keyword{database}
%% vim: syntax=tex

% $Id: fetch-methods.Rd,v 0.1 2008/07/23 02:38:31 psk Exp $
\name{fetch-methods}
\docType{methods}
\alias{fetch-methods}
\alias{fetch,PostgreSQLResult,numeric-method}
\alias{fetch,PostgreSQLResult,missing-method}
%\alias{fetch,PostgreSQLResult-method}
\title{
  Fetch records from a previously executed query
}
\description{
   This method is a straight-forward implementation of the corresponding
   generic function.
}
\section{Methods}{\describe{

\item{res}{
  an \code{PostgreSQLResult} object.
  }
\item{n}{
  maximum number of records to retrieve per fetch.
  Use \code{n = -1} to retrieve all pending records;
  use a value of \code{n = 0} for fetching the default number 
  of rows \code{fetch.default.rec} defined in the 
  \code{\link{PostgreSQL}} initialization invocation.
  }
\item{\dots }{currently not used.}
}
}
\details{
  The \code{RPostgreSQL} implementations retrieves only \code{n} records,
  and if \code{n} is missing it only returns up to \code{fetch.default.rec}
  as specified in the call to \code{\link{PostgreSQL}} (500 by default).
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
  \code{\link{PostgreSQL}},
  \code{\link[DBI]{dbConnect}},
  \code{\link[DBI]{dbSendQuery}},
  \code{\link[DBI]{dbGetQuery}},
  \code{\link[DBI]{dbClearResult}},
  \code{\link[DBI]{dbCommit}},
  \code{\link[DBI]{dbGetInfo}},
  \code{\link[DBI]{dbReadTable}}.
}
\examples{\dontrun{
drv <- dbDriver("PostgreSQL")
con <- dbConnect(drv, user = "ruser",password = "123456",dbname = "status")
res <- dbSendQuery(con, statement = paste(
                      "SELECT w.category, w.cost, p.type",
                      "FROM items w, sales P",
                      "WHERE w.category = p.type",
                      "ORDER BY w.cost"))
# we now fetch the first 100 records from the resultSet into a data.frame
data1 <- fetch(res, n = 100)   
dim(data1)

dbHasCompleted(res)

# let's get all remaining records
data2 <- fetch(res, n = -1)
}
}
\keyword{methods}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: PostgreSQLObject-class.Rd,v 0.1 2008/07/22 20:56:01 psk Exp $
\name{PostgreSQLObject-class}
\docType{class}
\alias{PostgreSQLObject-class}
\title{Class PostgreSQLObject}
\description{
   Base class for all PostgreSQL-specific DBI classes
}
\section{Objects from the Class}{
   A virtual Class: No objects may be created from it.
}
\section{Extends}{
Class \code{"DBIObject"}, directly.
Class \code{"dbObjectId"}, directly.
}
\section{Methods}{
  \describe{
    \item{\link{coerce}}{\code{signature(from = "PostgreSQLObject", to = "PostgreSQLriver")}: ... }
    \item{\link[DBI]{dbDataType}}{\code{signature(dbObj = "PostgreSQLObject")}: ... }
    \item{\link[DBI]{isSQLKeyword}}{\code{signature(dbObj = "PostgreSQLObject", name = "character")}: ... }
    \item{\link[DBI]{make.db.names}}{\code{signature(dbObj = "PostgreSQLObject", snames = "character")}: ... }
    \item{\link[DBI]{SQLKeywords}}{\code{signature(dbObj = "PostgreSQLObject")}: ... }
  }
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://developer.r-project.org/db/}.
}

\seealso{
 DBI base classes:

 \code{\link[DBI]{DBIObject-class}}
 \code{\link[DBI]{DBIDriver-class}}
 \code{\link[DBI]{DBIConnection-class}}
 \code{\link[DBI]{DBIResult-class}}

 PostgreSQL classes:

 \code{\link{PostgreSQLObject-class}}
 \code{\link{PostgreSQLDriver-class}}
 \code{\link{PostgreSQLConnection-class}}
 \code{\link{PostgreSQLResult-class}}

}

\examples{\dontrun{
drv <- dbDriver("PostgreSQL")
con <- dbConnect(drv, dbname = "template1")
}
}
\keyword{database}
\keyword{interface}
\keyword{classes}
% vim: syntax=tex

% $Id: PostgreSQL.Rd,v 0.1 2008/07/23 03:07:31 psk Exp $
\name{PostgreSQL}
\alias{PostgreSQL}
\title{
  Instantiate a PostgreSQL client from the current R or S-Plus session
}
\description{
This function creates and initializes a PostgreSQL client.
It returns an driver object that allows you to connect
to one or several PostgreSQL servers.
}
\usage{
PostgreSQL(max.con = 16, fetch.default.rec = 500, force.reload = FALSE)
}
\arguments{
\item{max.con }{
  Maximum number of connections that are intended to have open at one time.
  There's no intrinic limit, since strictly speaking this limit applies 
  to PostgreSQL \emph{servers}, but clients can have (at least in theory)
  more than this.  Typically there are at most a handful of open connections,
  thus the internal \code{RPostgreSQL} code uses a very simple linear search
  algorithm to manage its connection table.
  }
\item{fetch.default.rec}{
  number of records to fetch at one time from the database.
  (The \code{\link[DBI]{fetch}} method uses this number as a default.)
  }
\item{force.reload}{
  should the client code be reloaded (reinitialize)?
  Setting this to \code{TRUE} allows you to change
  default settings.  Notice that all connections should be closed
  before re-loading.
  }
}
\value{
An object \code{PostgreSQLDriver} that extends
\code{dbDriver} and 
\code{dbObjectId}.
This object is required to create connections
to one or several PostgreSQL database engines.
}
\section{Side Effects}{
The R/S-Plus client part of the database communication is initialized,
but note that connecting to the database engine needs to be done through
calls to \code{\link[DBI]{dbConnect}}.
}
\details{
This object is a singleton, that is, on subsequent invocations
it returns the same initialized object. 

This implementation allows you to connect
to multiple host servers and run multiple connections on each server
simultaneously.
}
\section{User authentication}{
The passed string can be empty to use all default parameters, or it can
contain one or more parameter settings separated by comma. Each
parameter setting is in the form parameter = "value". Spaces around the
equal sign are optional. 

The most important parameters are \code{user}, \code{password},
\code{host}, \code{dbname}, \code{port}, \code{tty} and \code{options}. 
}
\author{David A. James}
\section{References}{
See \url{https://cran.r-project.org/package=DBI}
for more details on the R/S-Plus database interface.

See the documentation at the PostgreSQL Web site
\url{https://www.postgresql.org} for details.
}
\seealso{
On database managers:

\code{\link[DBI]{dbDriver}}
\code{\link[DBI]{dbUnloadDriver}}

On connections, SQL statements and resultSets:

\code{\link[DBI]{dbConnect}}
\code{\link[DBI]{dbDisconnect}}
\code{\link[DBI]{dbSendQuery}}
\code{\link[DBI]{dbGetQuery}}
\code{\link[DBI]{fetch}}
\code{\link[DBI]{dbClearResult}}

On transaction management:

\code{\link[DBI]{dbCommit}}
\code{\link[DBI]{dbRollback}}

On meta-data:

\code{\link{summary}}
\code{\link[DBI]{dbGetInfo}}
\code{\link[DBI]{dbGetDBIVersion}}
\code{\link[DBI]{dbListTables}}
\code{\link[DBI]{dbListConnections}}
\code{\link[DBI]{dbListResults}}
\code{\link[DBI]{dbColumnInfo}}
\code{\link[DBI]{dbGetException}}
\code{\link[DBI]{dbGetStatement}}
\code{\link[DBI]{dbHasCompleted}}
\code{\link[DBI]{dbGetRowCount}}
\code{\link[DBI]{dbGetRowsAffected}}
}
\examples{\dontrun{
# create a PostgreSQL instance and create one connection.
> m <- dbDriver("PostgreSQL")
<PostgreSQLDriver:(4378)>

> con <- dbConnect(m, user="username", password="passwd", dbname="database_name")
> rs <- dbSendQuery(con, "select * sales where price < 10")
> df <- fetch(rs, n = 50)
> dbHasCompleted(rs)
[1] FALSE
> df2 <- fetch(rs, n = -1)
> dbHasCompleted(rs)
[1] TRUE
> dbClearResult(rs)
> dbListTables(con)    
}
}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: PostgreSQLConnection-class.Rd,v 0.1 2008/07/23 03:14:11 psk Exp $
\name{PostgreSQLConnection-class}
\docType{class}
\alias{PostgreSQLConnection-class}
\title{Class PostgreSQLConnection}
\description{PostgreSQLConnection class.}
\section{Generators}{
    The method \code{\link[DBI]{dbConnect}} is the main generator.
}
\section{Extends}{
Class \code{"DBIConnection"}, directly.
Class \code{"PostgreSQLObject"}, directly.
Class \code{"DBIObject"}, by class "DBIConnection".
Class \code{"dbObjectId"}, by class "PostgreSQLObject".
}
\section{Methods}{
  \describe{
    \item{\link{coerce}}{\code{signature(from = "PostgreSQLConnection", to = "PostgreSQLResult")}: ... }
    \item{\link[DBI]{dbBegin}}{\code{signature(conn = "PostgreSQLConnection")}: ... }
    \item{\link[DBI]{dbCallProc}}{\code{signature(conn = "PostgreSQLConnection")}: ... }
    \item{\link[DBI]{dbCommit}}{\code{signature(conn = "PostgreSQLConnection")}: ... }
    \item{\link[DBI]{dbConnect}}{\code{signature(drv = "PostgreSQLConnection")}: ... }
    \item{\link[DBI]{dbDisconnect}}{\code{signature(conn = "PostgreSQLConnection")}: ... }
    \item{\link[DBI]{dbExistsTable}}{\code{signature(conn = "PostgreSQLConnection", name = "character")}: ... }
    \item{\link[DBI]{dbGetException}}{\code{signature(conn = "PostgreSQLConnection")}: ... }
    \item{\link[DBI]{dbGetInfo}}{\code{signature(dbObj = "PostgreSQLConnection")}: ... }
    \item{\link[DBI]{dbGetQuery}}{\code{signature(conn = "PostgreSQLConnection", statement = "character")}: ... }
    \item{\link[DBI]{dbListFields}}{\code{signature(conn = "PostgreSQLConnection", name = "character")}: ... }
    \item{\link[DBI]{dbListResults}}{\code{signature(conn = "PostgreSQLConnection")}: ... }
    \item{\link[DBI]{dbListTables}}{\code{signature(conn = "PostgreSQLConnection")}: ... }
    \item{\link[DBI]{dbReadTable}}{\code{signature(conn = "PostgreSQLConnection", name = "character")}: ... }
    \item{\link[DBI]{dbRemoveTable}}{\code{signature(conn = "PostgreSQLConnection", name = "character")}: ... }
    \item{\link[DBI]{dbRollback}}{\code{signature(conn = "PostgreSQLConnection")}: ... }
    \item{\link[DBI]{dbSendQuery}}{\code{signature(conn = "PostgreSQLConnection", statement = "character")}: ... }
    \item{\link[DBI]{dbWriteTable}}{\code{signature(conn = "PostgreSQLConnection", name = "character", value = "data.frame")}: ... }
    \item{summary}{\code{signature(object = "PostgreSQLConnection")}: ... }
  }
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://developer.r-project.org/db/}.
}

\seealso{
 DBI base classes:

 \code{\link[DBI]{DBIObject-class}}
 \code{\link[DBI]{DBIDriver-class}}
 \code{\link[DBI]{DBIConnection-class}}
 \code{\link[DBI]{DBIResult-class}}

 PostgreSQL classes:

 \code{\link{PostgreSQLObject-class}}
 \code{\link{PostgreSQLDriver-class}}
 \code{\link{PostgreSQLConnection-class}}
 \code{\link{PostgreSQLResult-class}}

}

\examples{\dontrun{
drv <- dbDriver("PostgreSQL)
con <- dbConnect(drv, dbname = "template1")
}
}
\keyword{database}
\keyword{interface}
\keyword{classes}
% vim: syntax=tex

% $Id: dbReadTable-methods.Rd,v 0.1 2008/07/23 02:38:31 psk Exp $
\name{dbReadTable-methods}
\docType{methods}
\alias{dbReadTable-methods}
\alias{dbWriteTable-methods}
\alias{dbExistsTable-methods}
\alias{dbRemoveTable-methods}
\alias{dbReadTable,PostgreSQLConnection,character-method}
\alias{dbWriteTable,PostgreSQLConnection,character,character-method}
\alias{dbWriteTable,PostgreSQLConnection,character,data.frame-method}
\alias{dbExistsTable,PostgreSQLConnection,character-method}
\alias{dbRemoveTable,PostgreSQLConnection,character-method}
\title{
  Convenience functions for Importing/Exporting DBMS tables
}
\description{
  These functions mimic their R/S-Plus counterpart 
  \code{get},
  \code{assign},
  \code{exists}, 
  \code{remove}, and
  \code{objects},
  except that they generate code that gets remotely executed
  in a database engine.  
}
\section{Methods}{\describe{
\item{conn}{
  an \code{PostgreSQLConnection} database connection object.
  }
\item{name}{
  a character string specifying a table name.
  }
\item{value}{
  a data.frame (or coercible to data.frame).

  When the \code{value} is a character string, it is assumed to be a file name containing the data to be loaded;
  The implementation is INCOMPLETE and should not be used in current state.
  }
\item{row.names}{
  UNTESTED;

  in the case of \code{dbReadTable}, this argument can be a string or
  an index specifying the column in the DBMS table to be used as 
  \code{row.names} in the output data.frame (a \code{NULL}, \code{""}, or 0 
  specifies that no column should be used as \code{row.names} in the output).

  In the case of \code{dbWriteTable}, this argument should be a logical
  specifying whether the \code{row.names} should be output to the
  output DBMS table; if \code{TRUE}, an extra field whose name will be 
  whatever the R/S-Plus identifier \code{"row.names"} maps to the DBMS
  (see \code{\link[DBI]{make.db.names}}).
  }
\item{overwrite}{
  a logical specifying whether to overwrite an existing table
  or not.  Its default is \code{FALSE}.
  }
\item{append}{
  a logical specifying whether to append to an existing table
  in the DBMS.  Its default is \code{FALSE}.
  }
\item{allow.keywords}{
   \code{dbWriteTable} accepts a logical \code{allow.keywords} 
    to allow or prevent PostgreSQL reserved identifiers to be used as
    column names. By default it is \code{FALSE}.
   }
\item{dots}{
  optional arguments.

  When \code{dbWriteTable} is used to import data from a file, 
  you may optionally specify \code{header=}, \code{row.names=},
  \code{col.names=}, \code{sep=}, \code{eol=}, \code{field.types=}, 
  \code{skip=}, and \code{quote=}. NOT FULLY IMPLEMENTED YET.

  \code{header} is a logical indicating whether the first data line
  (but see \code{skip}) has a header or not.  If missing, it value
  is determined following \code{\link{read.table}} convention,
  namely, it is set to TRUE if and only if the first row has one
  fewer field that the number of columns.

  \code{row.names} is a logical to specify whether the first column
  is a set of row names.  If missing its default follows the 
  \code{\link{read.table}} convention.
  
  \code{col.names} a character vector with column names; column names are quoted to work as SQL identifiers.
  Thus, the column names are case sensitive and \code{\link[DBI]{make.db.names}} will NOT be used here.

  \code{sep=} specifies the field separator, and its default is \code{','}.

  \code{eol=} specifies the end-of-line delimiter, and its default is 
  \code{'\n'}.

  \code{skip} specifies number of lines to skip before reading the data,
  and it defaults to 0. 

  \code{field.types} is a list of named field SQL types where 
  \code{names(field.types)} provide the new table's column names 
  (if missing, field types are inferred using \code{\link[DBI]{dbDataType}}).
 }

}
}

\value{
   A \code{data.frame} in the case of \code{dbReadTable}; otherwise a logical 
   indicating whether the operation was successful.  
}
\note{
dbWriteTable creates additional column in the database, while dbReadTable 
reads that column by default. So, it is not symmetrical in its current implementation.
the backend raw_names may change in future versions.
}
\references{
   See the Database Interface definition document
   \code{DBI.pdf} in the base directory of this package
   or \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
  \code{\link{PostgreSQL}},
  \code{\link[DBI]{isSQLKeyword}},
  \code{\link[DBI]{dbDriver}},
  \code{\link[DBI]{dbConnect}},
  \code{\link[DBI]{dbSendQuery}},
  \code{\link[DBI]{dbGetQuery}},
  \code{\link[DBI]{fetch}},
  \code{\link[DBI]{dbCommit}},
  \code{\link[DBI]{dbGetInfo}},
  \code{\link[DBI]{dbListTables}},
  \code{\link[DBI]{dbReadTable}}.
}
\examples{\dontrun{
conn <- dbConnect("PostgreSQL", dbname = "wireless")
if(dbExistsTable(conn, "frame_fuel")){
   dbRemoveTable(conn, "frame_fuel")
   dbWriteTable(conn, "frame_fuel", fuel.frame)
}
if(dbExistsTable(conn, "RESULTS")){
   dbWriteTable(conn, "RESULTS", results2000, append = T)
else
   dbWriteTable(conn, "RESULTS", results2000)
}
}
}
\keyword{methods}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: PostgreSQLDriver-class.Rd,v 0.1 2008/07/22 20:56:01 psk Exp $
\name{PostgreSQLDriver-class}
\docType{class}
\alias{PostgreSQLDriver-class}
\title{Class PostgreSQLDriver}
\description{
   An PostgreSQL driver implementing the R/S-Plus database (DBI) API.
}
\section{Generators}{
   The main generators are \code{\link[DBI]{dbDriver}} and 
   \code{\link{PostgreSQL}}.
}
\section{Extends}{
Class \code{"DBIDriver"}, directly.
Class \code{"PostgreSQLObject"}, directly.
Class \code{"DBIObject"}, by class "DBIDriver".
Class \code{"dbObjectId"}, by class "PostgreSQLObject".
}
\section{Methods}{
  \describe{
    \item{\link{coerce}}{\code{signature(from = "PostgreSQLObject", to = "PostgreSQLDriver")}: ... }
    \item{\link[DBI]{dbConnect}}{\code{signature(drv = "PostgreSQLDriver")}: ... }
    \item{\link[DBI]{dbGetInfo}}{\code{signature(dbObj = "PostgreSQLDriver")}: ... }
    \item{\link[DBI]{dbListConnections}}{\code{signature(drv = "PostgreSQLDriver")}: ... }
    \item{\link[DBI]{dbUnloadDriver}}{\code{signature(drv = "PostgreSQLDriver")}: ... }
    \item{\link{summary}}{\code{signature(object = "PostgreSQLDriver")}: ... }
  }
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://developer.r-project.org/db/}.
}

\seealso{
 DBI base classes:

 \code{\link[DBI]{DBIObject-class}}
 \code{\link[DBI]{DBIDriver-class}}
 \code{\link[DBI]{DBIConnection-class}}
 \code{\link[DBI]{DBIResult-class}}

 PostgreSQL classes:

 \code{\link{PostgreSQLObject-class}}
 \code{\link{PostgreSQLDriver-class}}
 \code{\link{PostgreSQLConnection-class}}
 \code{\link{PostgreSQLResult-class}}
}

\examples{\dontrun{
drv <- dbDriver("PostgreSQL")
con <- dbConnect(drv, dbname="template1")
}
}
\keyword{database}
\keyword{interface}
\keyword{classes}
% vim: syntax=tex

% $Id: dbApply-methods.Rd,v 0.1 2008/08/10 18:04:01 psk Exp $
\name{dbApply-methods}
\docType{methods}
\alias{dbApply-methods}
\alias{dbApply,PostgreSQLResult-method}
\title{Apply R/S-Plus functions to remote groups of DBMS rows (experimental)}
\description{
  Applies R/S-Plus functions to groups of remote DBMS rows without
  bringing an entire result set all at once.  The result set
  is expected to be sorted by the grouping field.
}
\section{Methods}{\describe{
  \item{res}{a PostgreSQL result set (see \code{\link[DBI]{dbSendQuery}}).}
  \item{...}{any additional arguments to be passed to \code{FUN}.}
}
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or
  \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
   \code{\link{PostgreSQL}}
   \code{\link{postgresqlDBApply}}
   \code{\link[DBI]{dbSendQuery}}
   \code{\link[DBI]{fetch}}
}

\examples{\dontrun{
## compute quanitiles for each network agent
con <- dbConnect(PostgreSQL(), user="user", password="passwd",dbname="dbname")
rs <- dbSendQuery(con, 
             "select Agent, ip_addr, DATA from pseudo_data order by Agent")
out <- dbApply(rs, INDEX = "Agent", 
        FUN = function(x, grp) quantile(x$DATA, names=FALSE))
}
}
\keyword{programming}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: dbConnect-methods.Rd,v 0.1 2008/07/28 20:56:01 psk Exp $
\name{dbConnect-methods}
\docType{methods}
\alias{dbDisconnect-methods}
\alias{dbConnect-methods}
\alias{dbDisconnect,PostgreSQLConnection-method}
\alias{dbConnect,PostgreSQLDriver-method}
\alias{dbConnect,PostgreSQLConnection-method}
\alias{dbConnect,character-method}
\title{
  Create a connection object to an PostgreSQL DBMS
}
\description{
   These methods are straight-forward implementations of the corresponding
   generic functions.
}
\section{Methods}{\describe{
\item{drv}{
  an object of class \code{PostgreSQLDriver}, or
  the character string "PostgreSQL" or an \code{PostgreSQLConnection}.
  }
\item{conn}{
  an \code{PostgreSQLConnection} object as produced by \code{dbConnect}.
  }
\item{host}{Name or the numeric IPaddress of the host to connect to. If address is specified, it should be in the standard IPv4 address format, e.g., 172.28.40.9 or if your machine supports IPv6, you can also use those addresses. The default is to connect to localhost by a UNIX domain socket.
 }
\item{dbname}{The database name. Defaults to "", which is interpreted as PostgreSQL default.
  Most parameters are currently passed to PQsetdbLogin() as is. 
  If dbname contains an = sign or has a valid connection URI prefix, 
  it is taken as a conninfo for PQconnectdb() depending on the linked driver version.
  Refer to \url{https://www.postgresql.org/docs/current/libpq-connect.html} for detail.
 }
\item{user}{PostgreSQL user name to connect as. Defaults to be the same as the operating system name of the user running the application.
 }
\item{password}{Password to be used if the server demands password authentication.
 }
\item{port}{Port number to connect to at the server host.
 }
\item{tty}{Ignored (formerly, this specified where to send server debug output).
 }
\item{options}{Command-line options to be sent to the server.
 }
\item{forceISOdate}{logical if the communication of date (time stamp) from PostgreSQL is forced to ISO style at conection. This is an exception that is not really set by an option for PQsetdbLogin, but by issueing dbGetQuery(con, "set datestyle to ISO") after connection.
 }
}
}
\section{Side Effects}{
  A connection between R/S-Plus and an PostgreSQL  server is
  established.   The current implementation supports up to
  100 simultaneous connections.
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
  \code{\link{PostgreSQL}},
  \code{\link[DBI]{dbConnect}},
  \code{\link[DBI]{dbSendQuery}},
  \code{\link[DBI]{dbGetQuery}},
  \code{\link[DBI]{fetch}},
  \code{\link[DBI]{dbCommit}},
  \code{\link[DBI]{dbGetInfo}},
  \code{\link[DBI]{dbReadTable}}.
}
\examples{\dontrun{
# create an PostgreSQL instance and create one connection.
drv <- dbDriver("PostgreSQL")

# open the connection using user, passsword, etc., as
con <- dbConnect(drv, dbname = "postgres")

df <- dbGetQuery(con, statement = paste(
                      "SELECT itemCode, itemCost, itemProfit",
                      "FROM sales",
                      "SORT BY itemName"));
# Run an SQL statement by creating first a resultSet object
rs <- dbSendQuery(con, statement = paste(
                      "SELECT itemCode, itemCost, itemProfit",
                      "FROM sales",
                      "SORT BY itemName"));

# we now fetch records from the resultSet into a data.frame
df <- fetch(rs, n = -1)   # extract all rows
dim(df)
}
}
\keyword{methods}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: dbListTables-methods.Rd,v 0.1 2008/08/10 18:04:01 psk Exp $
\name{dbListTables-methods}
\docType{methods}
\alias{dbListTables-methods}
\alias{dbListFields-methods}
\alias{dbListConnections-methods}
\alias{dbListResults-methods}
\alias{dbListTables,PostgreSQLConnection-method}
\alias{dbListFields,PostgreSQLConnection,character-method}
\alias{dbListFields,PostgreSQLResult,missing-method}
\alias{dbListConnections,PostgreSQLDriver-method}
\alias{dbListResults,PostgreSQLConnection-method}
\title{
  List items from an PostgreSQL DBMS and from objects
}
\description{
   These methods are straight-forward implementations of the corresponding
   generic functions.
}
\section{Methods}{\describe{
  \item{drv}{an \code{PostgreSQLDriver}.}
  \item{conn}{an \code{PostgreSQLConnection}.}
  \item{name}{a character string with the table name.}
  \item{\dots}{currently not used.}
}
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or
  \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
  \code{\link{PostgreSQL}},
  \code{\link[DBI]{dbGetInfo}},
  \code{\link[DBI]{dbColumnInfo}},
  \code{\link[DBI]{dbDriver}},
  \code{\link[DBI]{dbConnect}},
  \code{\link[DBI]{dbSendQuery}}
}

\examples{\dontrun{
drv <- dbDriver("PostgreSQL")
# after working awhile...
for(con in dbListConnections(drv)){
   dbGetStatement(dbListResults(con))
}
}
}
\keyword{methods}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: dbApply.Rd,v d0.1 2008/08/10 18:04:01 psk Exp $
\name{dbApply}
\alias{dbApply}
\title{Apply R/S-Plus functions to remote groups of DBMS rows (experimental)}
\description{
  Applies R/S-Plus functions to groups of remote DBMS rows without
  bringing an entire result set all at once.  The result set
  is expected to be sorted by the grouping field.
}
%\usage{
%dbApply(res, ...)
%}
%\arguments{
%  \item{res}{a result set (see \code{\link[DBI]{dbSendQuery}}).}
%  \item{...}{any additional arguments to be passed to \code{FUN}.}
%}
\details{
  \code{dbApply} 
  This generic is meant to handle somewhat gracefully(?) large amounts 
  of data from the DBMS by bringing into R manageable chunks;
  the idea is that the data from individual groups can be handled by R, but
  not all the groups at the same time.  

  Currently, only the \code{\link{PostgreSQL}} driver implements a method
  (see the helper function \code{\link{postgresqlDBApply}}) for this
  generic function.
}
\value{
  A list with as many elements as there were groups in the
  result set.
}

\seealso{
   \code{\link{PostgreSQL}}
   \code{\link{postgresqlDBApply}}
   \code{\link[DBI]{dbSendQuery}}
   \code{\link[DBI]{fetch}}
}

\examples{\dontrun{
## compute quantiles for each network agent
con <- dbConnect(PostgreSQL(), user= "user", password="passwd", dbname="sample")
rs <- dbSendQuery(con, 
             "select Agent, ip_addr, DATA from pseudo_data order by Agent")
out <- dbApply(rs, INDEX = "Agent", 
        FUN = function(x, grp) quantile(x$DATA, names=FALSE))
}
}
\keyword{programming}% at least one, from doc/KEYWORDS
\keyword{interface}% __ONLY ONE__ keyword per line
\keyword{database}
% vim: syntax=tex

% $Id: isPostgresqlIdCurrent.Rd,v 0.1 2008/07/23 02:38:31 psk Exp $
\name{isPostgresqlIdCurrent}
\alias{isPostgresqlIdCurrent}
\title{
  Check whether a database handle object is valid or not
}
\description{
Support function that verifies that an object holding a reference
to a foreign object is still valid for communicating with the RDBMS
}
\usage{
isPostgresqlIdCurrent(obj)
}
\arguments{
\item{obj}{
any \code{dbObject} (e.g., \code{dbDriver},
\code{dbConnection}, \code{dbResult}).
}
}
\value{
a logical scalar.
}
\details{
\code{dbObjects} are R/S-Plus remote references to foreign objects.
This introduces differences to the object's semantics such as
persistence (e.g., connections may be closed unexpectedly),
thus this function provides a minimal verification to ensure 
that the foreign object being referenced can be contacted.
}
\seealso{
\code{\link[DBI]{dbDriver}}
\code{\link[DBI]{dbConnect}}
\code{\link[DBI]{dbSendQuery}}
\code{\link[DBI]{fetch}}
}
\examples{\dontrun{
cursor <- dbSendQuery(con, sql.statement)
isIdCurrent(cursor)
}
}
\keyword{interface}
\keyword{database}
% docclass is function
% vim: syntax=tex

% $Id: summary-methods.Rd,v 0.1 2008/08/10 15:09:01 psk $
\name{summary-methods}
\docType{methods}
\alias{coerce-methods}
\alias{summary-methods}
\alias{format-methods}
\alias{show-methods}
\alias{coerce,dbObjectId,integer-method}
\alias{coerce,dbObjectId,numeric-method}
\alias{coerce,dbObjectId,character-method}
\alias{coerce,PostgreSQLObject,PostgreSQLDriver-method}
\alias{coerce,PostgreSQLConnection,PostgreSQLResult-method}
\alias{coerce,PostgreSQLConnection,PostgreSQLDriver-method}
\alias{coerce,PostgreSQLResult,PostgreSQLConnection-method}
\alias{coerce,PostgreSQLResult,PostgreSQLDriver-method}
\alias{format,dbObjectId-method}
\alias{print,dbObjectId-method}
\alias{show,dbObjectId-method}
\alias{summary,PostgreSQLObject-method}
\alias{summary,PostgreSQLDriver-method}
\alias{summary,PostgreSQLConnection-method}
\alias{summary,PostgreSQLResult-method}
\title{Summarize an PostgreSQL object}
\description{
   These methods are straight-forward implementations of the corresponding
   generic functions.
}
\section{Methods}{\describe{

\item{object = "DBIObject"}{
   Provides relevant metadata information on \code{object}, 
   for instance, the PostgreSQL server file, the
   SQL statement associated with a result set, etc.  
   }
\item{from}{object to be coerced}
\item{to}{coercion class}
\item{x}{object to \code{format} or \code{print} or \code{show}}
}
}
\keyword{methods}
\keyword{database}
\keyword{interface}
% vim: syntax=tex

% $Id: PostgreSQLResult-class.Rd,v 0.1 2008/07/22 20:56:01 psk Exp $
\name{PostgreSQLResult-class}
\docType{class}
\alias{PostgreSQLResult-class}
\title{Class PostgreSQLResult}
\description{
   PostgreSQL's query results class.  This classes encapsulates the
   result of an SQL statement (either \code{select} or not).
}
\section{Generators}{
   The main generator is \code{\link[DBI]{dbSendQuery}}.
}
\section{Extends}{
Class \code{"DBIResult"}, directly.
Class \code{"PostgreSQLObject"}, directly.
Class \code{"DBIObject"}, by class "DBIResult".
Class \code{"dbObjectId"}, by class "PostgreSQLObject".
}
\section{Methods}{
  \describe{
    \item{\link{coerce}}{\code{signature(from = "PostgreSQLConnection", to = "PostgreSQLResult")}: ... }
    \item{\link[DBI]{dbClearResult}}{\code{signature(res = "PostgreSQLResult")}: ... }
    \item{\link[DBI]{dbColumnInfo}}{\code{signature(res = "PostgreSQLResult")}: ... }
    \item{\link[DBI]{dbGetException}}{\code{signature(conn = "PostgreSQLResult")}: ... }
    \item{\link[DBI]{dbGetInfo}}{\code{signature(dbObj = "PostgreSQLResult")}: ... }
    \item{\link[DBI]{dbGetRowCount}}{\code{signature(res = "PostgreSQLResult")}: ... }
    \item{\link[DBI]{dbGetRowsAffected}}{\code{signature(res = "PostgreSQLResult")}: ... }
    \item{\link[DBI]{dbGetStatement}}{\code{signature(res = "PostgreSQLResult")}: ... }
    \item{\link[DBI]{dbHasCompleted}}{\code{signature(res = "PostgreSQLResult")}: ... }
    \item{\link[DBI]{dbListFields}}{\code{signature(conn = "PostgreSQLResult", name = "missing")}: ... }
    \item{\link[DBI]{fetch}}{\code{signature(res = "PostgreSQLResult", n = "numeric")}: ... }
    \item{\link[DBI]{fetch}}{\code{signature(res = "PostgreSQLResult", n = "missing")}: ... }
    \item{\link{summary}}{\code{signature(object = "PostgreSQLResult")}: ... }
  }
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://developer.r-project.org/db/}
}

\seealso{
 DBI base classes:

 \code{\link[DBI]{DBIObject-class}}
 \code{\link[DBI]{DBIDriver-class}}
 \code{\link[DBI]{DBIConnection-class}}
 \code{\link[DBI]{DBIResult-class}}

 PostgreSQL classes:

 \code{\link{PostgreSQLObject-class}}
 \code{\link{PostgreSQLDriver-class}}
 \code{\link{PostgreSQLConnection-class}}
 \code{\link{PostgreSQLResult-class}}

}

\examples{\dontrun{
drv <- dbDriver("PostgreSQL")
con <- dbConnect(drv, dbname = "template1")
## rs is the result set
rs <- dbSendQuery(con,"select * from sales")
## display the first three values of result set
fetch(rs,n=3)
}
}
\keyword{database}
\keyword{interface}
\keyword{classes}
% vim: syntax=tex

% $Id: postgresqlDBApply.Rd,v 0.1 2008/07/23 03:09:11 psk Exp $
\name{postgresqlDBApply}
\alias{postgresqlDBApply}
\title{Apply R/S-Plus functions to remote groups of DBMS rows (experimental)}
\description{
  Applies R/S-Plus functions to groups of remote DBMS rows without
  bringing an entire result set all at once.  The result set
  is expected to be sorted by the grouping field.
}
\usage{
postgresqlDBApply(res, INDEX, FUN = stop("must specify FUN"),
         begin = NULL, 
         group.begin =  NULL, 
         new.record = NULL, 
         end = NULL, 
         batchSize = 100, maxBatch = 1e6, 
         ..., simplify = TRUE)
}
\arguments{
  \item{res}{a result set (see \code{\link[DBI]{dbSendQuery}}).}
  \item{INDEX}{a character or integer specifying the field name or
               field number that defines the various groups.}
  \item{FUN}{a function to be invoked upon identifying the last
               row from every group. This function will be passed
               a data frame holding the records of the current group, 
               a character string with the group label, plus any
               other arguments passed to \code{dbApply} as \code{"..."}.}
  \item{begin}{a function of no arguments to be invoked just prior to 
               retrieve the first row from the result set.}
  \item{end}{a function of no arguments to be invoked just after retrieving 
               the last row from the result set.}
  \item{group.begin}{a function of one argument (the group label) to be 
               invoked upon identifying a row from a new group}.
  \item{new.record}{a function to be invoked as each individual record
               is fetched.  The first argument to this function is a
               one-row data.frame holding the new record.}
  \item{batchSize}{the default number of rows to bring from the remote 
                result set. If needed, this is automatically extended
                to hold groups bigger than \code{batchSize}.}
  \item{maxBatch}{the absolute maximum of rows per group that may
                be extracted from the result set.}
  \item{...}{any additional arguments to be passed to \code{FUN}.}
  \item{simplify}{Not yet implemented}
}
\details{
  \code{dbApply} 
  This function is meant to handle somewhat gracefully(?) large amounts 
  of data from the DBMS by bringing into R manageable chunks (about 
  \code{batchSize} records at a time, but not more than \code{maxBatch}); 
  the idea is that the data from individual groups can be handled by R, but
  not all the groups at the same time.  
  
  The PostgreSQL implementation \code{postgresqlDBApply} allows us to register R
  functions that get invoked
  when certain fetching events occur. These include the ``begin'' event
  (no records have been yet fetched), ``begin.group'' (the record just 
  fetched belongs to a new group), ``new record'' (every fetched record
  generates this event), ``group.end'' (the record just fetched was the
  last row of the current group), ``end'' (the very last record from the
  result set). Awk and perl programmers will find this paradigm very
  familiar (although SAP's ABAP language is closer to what we're doing).
}
\value{
  A list with as many elements as there were groups in the
  result set.
}
\note{This is an experimental version implemented only in R (there are
plans, time permitting, to implement it in S-Plus).

The terminology that we're using is closer to SQL than R.  In R
what we're referring to ``groups'' are the individual levels of
a factor (grouping field in our terminology).
}

\seealso{\code{\link{PostgreSQL}}, \code{\link[DBI]{dbSendQuery}}, \code{\link[DBI]{fetch}}.}

\examples{\dontrun{
drv <- dbDriver(RPostgreSQL)
con <- dbConnect(drv, user ="usrname", password="pword", dname="database")
res <- dbSendQuery(con, 
             "select Agent, ip_addr, DATA from pseudo_data order by Agent")
out <- dbApply(res, INDEX = "Agent", 
        FUN = function(x, grp) quantile(x$DATA, names=FALSE))
}
}
\keyword{programming}% at least one, from doc/KEYWORDS
\keyword{interface}% __ONLY ONE__ keyword per line
\keyword{database}
% vim: syntax=tex

% $Id: dbCallProc-methods.Rd,v 0.1 2008/08/10 18:04:01 psk Exp $
\name{dbCallProc-methods}
\docType{methods}
\alias{dbCallProc-methods}
\alias{dbCallProc,PostgreSQLConnection-method}
\title{
  Call an SQL stored procedure
}
\description{
  Not yet implemented.
}
\section{Methods}{\describe{
\item{conn}{
  a \code{PostgreSQLConnection} object.
  }
\item{\dots }{
  additional arguments are passed to the implementing method.
  }
}
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
   \code{\link{PostgreSQL}},
   \code{\link[DBI]{dbConnect}},
   \code{\link[DBI]{dbSendQuery}},
   \code{\link[DBI]{dbGetQuery}},
   \code{\link[DBI]{fetch}},
   \code{\link[DBI]{dbCommit}},
   \code{\link[DBI]{dbGetInfo}},
   \code{\link[DBI]{dbReadTable}}.
}
\keyword{methods}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: dbSetDataMappings-methods.Rd, V 0.1 2008/07/23 02:38:31 psk Exp $
\name{dbSetDataMappings-methods}
\docType{methods}
\alias{dbSetDataMappings-methods}
\alias{dbSetDataMappings,PostgreSQLResult,data.frame-method}
\title{
  Set data mappings between PostgreSQL and R/S-Plus
}
\description{
  Not yet implemented
}
\section{Methods}{\describe{
\item{res}{
  a \code{PostgreSQLResult} object as returned by \code{dbSendQuery}.
  }
\item{flds}{
  a data.frame with field descriptions as returned by 
  \code{\link[DBI]{dbColumnInfo}}.
  }
\item{\dots }{
  any additional arguments are passed to the implementing method.
  }
}
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
  \code{\link{PostgreSQL}},
  \code{\link[DBI]{dbSendQuery}},
  \code{\link[DBI]{fetch}},
  \code{\link[DBI]{dbColumnInfo}}.
}
\examples{\dontrun{
makeImage <- function(x) {
  .C("make_Image", as.integer(x), length(x))
}

res <- dbSendQuery(con, statement)
flds <- dbColumnInfo(res)
flds[3, "Sclass"] <- makeImage

dbSetDataMappings(rs, flds)

im <- fetch(rs, n = -1)
}
}
\keyword{methods}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: postgresqlSupport.Rd,v 0.1 2008/07/23 02:23:01 psk Exp $
\name{postgresqlSupport}
\alias{postgresqlInitDriver}            % driver-related function
\alias{postgresqlDriverInfo}
\alias{postgresqlDescribeDriver}
\alias{postgresqlCloseDriver}           % connection-related
\alias{postgresqlNewConnection}
\alias{postgresqlCloneConnection}
\alias{postgresqlConnectionInfo}
\alias{postgresqlDescribeConnection}
\alias{postgresqlCloseConnection}
\alias{postgresqlExecStatement}         % result-related
\alias{postgresqlFetch}
\alias{postgresqlQuickSQL}
\alias{postgresqlResultInfo}
\alias{postgresqlDescribeResult}
\alias{postgresqlCloseResult}
\alias{postgresqlDescribeFields}
\alias{postgresqlReadTable}
\alias{postgresqlWriteTable}            % convenience functions
\alias{postgresqlpqExec}
\alias{postgresqlCopyIn}
\alias{postgresqlCopyInDataframe}
\alias{postgresqlgetResult}
\alias{postgresqlEscapeStrings}
\alias{postgresqlEscapeBytea}
\alias{postgresqlUnescapeBytea}
\alias{postgresqlQuoteId}
\alias{postgresqlTableRef}
\alias{postgresqlImportFile}
\alias{postgresqlDataType}
\alias{postgresqlTransactionStatement}   % transaction management 
\alias{.PostgreSQLPkgName}              % constants
\alias{.PostgreSQLPkgVersion}
\alias{.PostgreSQLPkgRCS}
\alias{.PostgreSQL.NA.string}
\alias{.PostgreSQLSQLKeywords}
\alias{.conflicts.OK}
%\non_function{}
\title{Support Functions}
\description{
   These functions are the workhorse behind the RPostgreSQL package, but
   users need not invoke these directly.  For details see \code{\link{PostgreSQL}}.
}
\usage{
   ## PostgreSQLDriver-related
   postgresqlInitDriver(max.con=16, fetch.default.rec = 500, force.reload=FALSE)
   postgresqlDriverInfo(obj, what, ...)
   postgresqlDescribeDriver(obj, verbose = FALSE, ...)
   postgresqlCloseDriver(drv, ...)

   ## PostgreSQLConnection-related
   postgresqlNewConnection(drv, user, password, host, dbname, port, 
			tty, options, forceISOdate = TRUE)
   postgresqlCloneConnection(con, ...)
   postgresqlConnectionInfo(obj, what, ...)
   postgresqlDescribeConnection(obj, verbose = FALSE, ...)
   postgresqlCloseConnection(con, ...)

   ## PostgreSQLResult-related
   postgresqlExecStatement(con, statement, params, ...)
   postgresqlFetch(res, n=0, ...)
   postgresqlQuickSQL(con, statement, ...)
   postgresqlResultInfo(obj, what, ...)
   postgresqlDescribeResult(obj, verbose = FALSE, ...)
   postgresqlCloseResult(res, ...)
   postgresqlDescribeFields(res, ...)

   ## data mappings, convenience functions, and extensions
   postgresqlDataType(obj, ...)
   postgresqlReadTable(con, name, row.names = "row.names", check.names = TRUE, ...)
   postgresqlWriteTable(con, name, value, field.types, row.names = TRUE,
      overwrite=FALSE, append=FALSE, ..., allow.keywords = FALSE)
   postgresqlpqExec(con, statement)
   postgresqlCopyIn(con, filename)
   postgresqlgetResult(con)
   postgresqlEscapeStrings(con, preescapedstring)
   postgresqlEscapeBytea(con, raw_data)
   postgresqlUnescapeBytea(escapedbytea)
   postgresqlQuoteId(identifiers)
   postgresqlTableRef(identifiers)
   postgresqlImportFile(con, name, value, field.types, overwrite=FALSE,
      append=FALSE, header, row.names, nrows=50, sep=",", eol="\n", 
      skip = 0, quote="\"", ...)

   ## Transaction Management
   postgresqlTransactionStatement(con, statement)
}
\arguments{
\item{max.con}{
   positive integer specifying maximum number of open connections.
   The current default of 10 is hardcoded in the C code.
}
\item{fetch.default.rec}{
   default number of rows to fetch (move to R/S-Plus).  This default is
   used in \code{postgresqlFetch}.
   The default is 500.
}
\item{force.reload}{
   logical indicating whether to re-initialize the driver.  This may be
   useful if you want to change the defaults (e.g., \code{fetch.default.rec}).
   Note that the driver is a singleton (subsequent inits just returned the
   previously initialized driver, thus this argument).
}
\item{obj}{
   any of the PostgreSQL DBI objects (e.g., \code{PostgreSQLConnection},
   \code{PostgreSQLResult}).
}
\item{what}{
   character vector of metadata to extract, e.g., "version", "statement",
   "isSelect".
}
\item{verbose}{
   logical controlling how much information to display.
   Defaults to \code{FALSE}.
}
\item{drv}{
   an \code{PostgreSQLDriver} object as produced by \code{postgresqlInitDriver}.
}
\item{con}{
   an \code{PostgreSQLConnection} object as produced by \code{postgresqlNewConnection}
   and \code{postgresqlCloneConnection}.
}
\item{res}{
   an \code{PostgreSQLResult} object as produced by by \code{postgresqlExecStatement}
   and \code{postgresqlgetResult}.
}
\item{user}{
   a character string with the PostgreSQL's user name.
}
\item{password}{
   character string with the PostgreSQL's password.
}
\item{dbname}{ character string with the PostgreSQL database name.
}
\item{host}{
   character string with the name (or IP address) of the machine
   hosting the database. Default is \code{""}, which is
   interpreted as \code{localhost} by the PostgreSQL's API.
}
\item{port}{
   (optional) positive integer specifying the TCP port number that
   the PostgreSQL server is listening to.  Consult the PostgreSQL documentation
   for details.
}
\item{tty}{
Ignored (formerly, this specified where to send server debug output)
}
\item{options}{
Command-line options to be sent to the server
}
\item{forceISOdate}{
   logical indicating whether "set datestyle to ISO"
   is issued upon connection. Although this is made as an option, the
   conversion needs to be ISO style for proper conversion of the date datatype.
}
\item{force}{
   logical indicating whether to close a connection that has open
   result sets.  The default is \code{FALSE}.
}
\item{statement}{
   character string holding one (and only one) SQL statement.
}
\item{params}{
   actual values that is written as parameters in the statement.
}
\item{n}{
   number of rows to fetch from the given result set. A value of -1
   indicates to retrieve all the rows.  The default of 0 specifies
   to extract whatever the \code{fetch.default.rec} was specified
   during driver initialization \code{postgresqlInit}.
}
\item{name}{
   character vector of names (table names, fields, keywords).
}
\item{value}{
   a data.frame.
}
\item{field.types}{
   a list specifying the mapping from R/S-Plus fields in the data.frame
   \code{value} to SQL data types.  The default is
   \code{sapply(value,SQLDataType)}, see \code{PostgreSQLSQLType}.
}
\item{header}{
  logical, does the input file have a header line?  Default is the
  same heuristic used by \code{read.table}, i.e., TRUE if the first
  line has one fewer column that the second line.
}
\item{row.names}{
   a logical specifying whether to prepend the \code{value} data.frame 
   row names or not.  The default is \code{TRUE}.
}
\item{check.names}{
   a logical specifying whether to convert DBMS field names into
   legal S names. Default is \code{TRUE}.
}
\item{overwrite}{
   logical indicating whether to replace the table \code{name} with
   the contents of the data.frame \code{value}.
   The defauls is \code{FALSE}.
}
\item{append}{
   logical indicating whether to append \code{value} to the existing
   table \code{name}.
}
\item{nrows}{
  number of lines to rows to import using \code{read.table} from the
  input file to create the proper table definition. Default is 50.
}
\item{sep}{field separator character.}
\item{eol}{end-of-line separator.}
\item{skip}{number of lines to skip before reading data in the input file.}
\item{quote}{the quote character used in the input file (defaults to \code{\"}.}
\item{allow.keywords}{
   logical indicating whether column names that happen to be PostgreSQL
   keywords be used as column names in the resulting relation (table)
   being written.  Defaults to \code{FALSE}, forcing \code{postgresqlWriteTable}
   to modify column names to make them legal PostgreSQL identifiers.
   }
\item{preescapedstring}{character string to be escaped}
\item{raw_data}{RAW data to be escaped}
\item{escapedbytea}{'escaped' or hex encoded binary data as character}
\item{identifiers}{one or more character strings to be used as identfier in SQL statement}
\item{filename}{character string indicating the file which contains the data to be copied to the PostgreSQL backend}
\item{\dots}{
   placeholder for future use.
}
}
\value{
   \code{postgresqlInitDriver} returns an \code{PostgreSQLDriver} object.

   \code{postgresqlDriverInfo} returns a list of name-value metadata pairs.

   \code{postgresqlDescribeDriver} returns \code{NULL} (displays the object's
   metadata).

   \code{postgresqlCloseDriver} returns a logical indicating whether the
   operation succeeded or not.

   \code{postgresqlNewConnection} returns an \code{PostgreSQLConnection} object.

   \code{postgresqlCloneConnection} returns an \code{PostgreSQLConnection} object.

   \code{postgresqlConnectionInfo}returns a list of name-value metadata pairs.

   \code{postgresqlDescribeConnection} returns \code{NULL} (displays the
   object's metadata).

   \code{postgresqlCloseConnection} returns a logical indicating whether the
   operation succeeded or not.

   \code{postgresqlExecStatement} returns an \code{PostgreSQLResult} object.

   \code{postgresqlFetch} returns a data.frame.

   \code{postgresqlQuickSQL} returns either a data.frame if the \code{statement} is
   a \code{select}-like or NULL otherwise.

   \code{postgresqlDescribeResult} returns \code{NULL} (displays the object's
   metadata).
   
   \code{postgresqlCloseResult} returns a logical indicating whether the
   operation succeeded or not.
   
   \code{postgresqlDescribeFields} returns a data.frame with one row per field
   with columns \code{name}, \code{Sclass}, \code{type}, \code{len}, 
   \code{precision}, \code{scale}, and \code{nullOK} which fully describe
   each field in a result set.  Except for \code{Sclass} (which shows the
   mapping of the field type into an R/S-Plus class) all the information
   pertains to PostgreSQL's data storage attributes.

   \code{postgresqlReadTable}  returns a data.frame with the contents of the
   DBMS table.
   
   \code{postgresqlWriteTable}  returns a logical indicating whether the
   operation succeeded or not.

   \code{postgresqlpqExec}  returns \code{NUL} (executes the statement
   but does not try to get result.  This is called internally
   from \code{postgresqlWriteTable} before \code{postgresqlCopyInDataframe}

   \code{postgresqlCopyIn}  returns \code{NULL} (copies the content 
   of the file through the socket connection to postgresql backend.
   This should be used just after COPY tablename FROM STDIN query. This is not used now.)

   \code{postgresqlCopyInDataframe}  returns \code{NULL} (copies the content 
   of the dataframe through the socket connection to postgresql backend. 
   Strings are encoded as UTF-8 for transfer.  The client_encoding should be set to UTF-8.
   This should be used just after COPY tablename FROM STDIN query.)

   \code{postgresqlgetResult}  returns an \code{PostgreSQLResult} object.  
   This is called after completion of execution of \code{postgresqlpqExec}.

   \code{postgresqlEscapeStrings} returns a character string which is escaped properly
   so that it can be surrounded with a single quote and used as literal in SQL.
   The escape procedure is dependent on the character encoding of the connection.

   \code{postgresqlEscapeBytea} returns a character string which represents the
   raw data proper escape so that it can be surrounded 
   with a single quote and used as literal in SQL. Note that on 8.X connection E prefix
   should exist before the first quote. However this changes the behaviour in 9.0, where
   the E should not exist.

   \code{postgresqlUnescapeBytea} returns a raw data specified by the character string.
   The character string should contain the 'escaped' bytea or hex encoded bytea
   that was output from the database server.

   \code{postgresqlQuoteId} returns a character string which is quoted as identifier. Returns vector on vector arguemnt.

   \code{postgresqlTableRef} returns a character string which is quoted as identifier. 
    Reterns a charcter string concatenated with "." so that "dbname"."schemaname"."tablename" reference
    is created upon c("dbname", "schemaname", "tablename") arguemnt.
   
   \code{postgresqlDataType} retuns a character string with the closest
   
   \code{postgresqlResultInfo} returns a list of name-value metadata pairs.

   \code{postgresqlTransactionStatement} returns a logical indicating whether the
   operation succeeded or not.
}
\section{Constants}{
\code{.PostgreSQLPkgName} (currently \code{"RPostgreSQL"}),
\code{.PostgreSQLPkgVersion} (the R package version),
\code{.PostgreSQLPkgRCS} (the RCS revision),
\code{.PostgreSQL.NA.string} (character that PostgreSQL uses to
denote \code{NULL} on input),
\code{.PostgreSQLSQLKeywords} (a lot!)
\code{.conflicts.OK}.
%\non_function{}
}
\keyword{datasets}
\keyword{interface}
\keyword{database}
%\keyword{internal}
% vim:syntax=tex

% $Id: dbDataType-methods.Rd,v  0.1 2008/08/10 18:04:01 psk Exp $
\name{dbDataType-methods}
\docType{methods}
\alias{dbDataType-methods}
\alias{dbDataType,PostgreSQLObject-method}
\title{
  Determine the SQL Data Type of an S object
}
\description{
  This method is a straight-forward implementation of the corresponding
  generic function.
}
\section{Methods}{
  \describe{
    \item{dbObj}{
      any \code{PostgreSQLObject} object, e.g., \code{PostgreSQLDriver},
      \code{PostgreSQLConnection}, \code{PostgreSQLResult}.
    }
    \item{obj}{
      R/S-Plus object whose SQL type we want to determine.
    }
    \item{\dots}{
      any other parameters that individual methods may need.
    }
  }
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
  \code{\link[DBI]{isSQLKeyword}}
  \code{\link[DBI]{make.db.names}}
}
\examples{\dontrun{
data(quakes)
drv <- dbDriver("PostgreSQL")
sql.type <- dbDataType(drv, quakes)
}
}
\keyword{methods}
\keyword{interface}
\keyword{database}
% docclass is function
% Converted by Sd2Rd version 1.15.2.1.
% vim: syntax=tex

% $Id: dbDriver-methods.Rd,v 0.1 2008/07/28 01:09:05 psk Exp $
\name{dbDriver-methods}
\docType{methods}
\alias{dbDriver-methods}
\alias{dbUnloadDriver-methods}
\alias{dbDriver,character-method}
\alias{dbUnloadDriver,PostgreSQLDriver-method}
\title{
  PostgreSQL implementation of the Database Interface (DBI) classes
  and drivers 
}
\description{
  PostgreSQL driver initialization and closing
}
\section{Methods}{\describe{
\item{drvName}{
  character name of the driver to instantiate.
  }
\item{drv}{
  an object that inherits from \code{PostgreSQLDriver} as created by
  \code{dbDriver}.
}
\item{max.con}{optional integer requesting the maximum number of
simultanous connections (may be up to 100)}.
\item{fetch.default.rec}{default number of records to retrieve per fetch.
Default is 500.  This may be overridden in calls to \code{\link[DBI]{fetch}}
with the \code{n=} argument.}
\item{force.reload}{optional logical used to force re-loading or recomputing
the size of the connection table. Default is \code{FALSE}.}
\item{...}{currently unused.}
}
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or
  \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
  \code{\link{PostgreSQL}},
  \code{\link[DBI]{dbConnect}},
  \code{\link[DBI]{dbSendQuery}},
  \code{\link[DBI]{dbGetQuery}},
  \code{\link[DBI]{fetch}},
  \code{\link[DBI]{dbCommit}},
  \code{\link[DBI]{dbGetInfo}},
  \code{\link[DBI]{dbListTables}},
  \code{\link[DBI]{dbReadTable}}.
}
\examples{\dontrun{

# create an PostgreSQL instance and set 10000 of rows per fetch.
library(RPostgreSQL)
drv <- dbDriver("PostgreSQL", fetch.default.records=10000)

# Connecting to PostgreSQL with the specified parameters
con <- dbConnect(drv,user="usrname",password="passwd",dbname="postgres")

# Running the query to obtain the resultset
rs <- dbSendQuery(con, "select * from cities where population > 5000")

# fetch records into a dataframe.
# n = 50 fetched fifty records
df <- fetch(rs, n = 50)
# n = -1 fetches all the remaining records available
df2 <- fetch(rs, n = -1)

# Clearing the result set
dbClearResult(rs)

#This returns a character vector (possibly of zero-length)
# table names available on the con connection.
dbListTables(con)
}
}
\keyword{methods}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: dbSendQuery-methods.Rd,v 0.1 2008/07/23 02:38:31 psk Exp $
\name{dbSendQuery-methods}
\docType{methods}
\alias{dbSendQuery-methods}
\alias{dbGetQuery-methods}
\alias{dbClearResult-methods}
\alias{dbGetException-methods}
\alias{dbSendQuery,PostgreSQLConnection,character-method}
\alias{dbGetQuery,PostgreSQLConnection,character-method}
\alias{dbClearResult,PostgreSQLResult-method}
\alias{dbGetException,PostgreSQLConnection-method}
\alias{dbGetException,PostgreSQLResult-method}
\title{
  Execute a statement on a given database connection
}
\description{
   These methods are straight-forward implementations of the corresponding
   generic functions.
   However, for complex data like array are just transferred as a string instead of the corresponding
   vector in R.  This behavior will change in future releases, and the author is advised not to rely on it.
}
\section{Methods}{\describe{
\item{conn}{
  an \code{PostgreSQLConnection} object.
  }
\item{statement}{a character vector of length 1 with the SQL statement.}
\item{res}{an \code{PostgreSQLResult} object.}
\item{\dots }{additional parameters.}
}
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
  \code{\link{PostgreSQL}},
  \code{\link[DBI]{dbDriver}},
  \code{\link[DBI]{dbConnect}},
  \code{\link[DBI]{fetch}},
  \code{\link[DBI]{dbCommit}},
  \code{\link[DBI]{dbGetInfo}},
  \code{\link[DBI]{dbReadTable}}.
}
\examples{\dontrun{
drv <- dbDriver("PostgreSQL")
con <- dbConnect(drv, "usr", "password", "dbname")
res <- dbSendQuery(con, "SELECT * from sales")
data <- fetch(res, n = -1)
# alternatively, use dbGetQuery
data <- dbGetQuery(con, "SELECT * from sales")
}
}
\keyword{methods}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: dbCommit-methods.Rd,v 0.1 2008/08/10 18:04:01 psk Exp $
\name{dbCommit-methods}
\docType{methods}
\alias{dbBegin-method}
\alias{dbCommit-method}
\alias{dbRollback-method}
\alias{PostgreSQLConnection-method}
\alias{dbBegin,PostgreSQLConnection-method}
\alias{dbCommit,PostgreSQLConnection-method}
\alias{dbRollback,PostgreSQLConnection-method}
\title{
  DBMS Transaction Management
}
\description{
  Transaction related commands.
  Start a transaction, commit or roll back the current transaction 
  in an PostgreSQL connection.
  \code{dbBegin} starts a transaction.
  \code{dbCommit} and \code{dbRollback} commit and rollback the
  transaction, respectively.

}
\section{Methods}{\describe{
\item{conn}{a \code{PostgreSQLConnection} object, as produced by the function
  \code{dbConnect}.}
\item{\dots }{currently unused.}
}
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or
  \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
   \code{\link{PostgreSQL}},
   \code{\link[DBI]{dbBegin}},
   \code{\link[DBI]{dbConnect}},
   \code{\link[DBI]{dbSendQuery}},
   \code{\link[DBI]{dbGetQuery}},
   \code{\link[DBI]{fetch}},
   \code{\link[DBI]{dbCommit}},
   \code{\link[DBI]{dbGetInfo}},
   \code{\link[DBI]{dbReadTable}}.
}
\examples{\dontrun{
drv <- dbDriver("PostgreSQL")
con <- dbConnect(drv, dbname="postgres")
dbGetQuery(con, "select count(*) from sales")

dbBegin(con)
rs <- dbSendQuery(con, 
      "Delete * from sales as p where p.cost>10")
if(dbGetInfo(rs, what = "rowsAffected") > 250){
  warning("Rolling back transaction")
  dbRollback(con)
}else{
  dbCommit(con)
}

dbGetQuery(con, "select count(*) from sales")
dbDisconnect(con)
}
}

\keyword{methods}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: dbGetInfo-methods.Rd,v 0.1 2008/08/10 18:04:01 psk Exp $
\name{dbGetInfo-methods}
\docType{methods}
\alias{dbGetInfo}
\alias{dbGetDBIVersion-methods}
\alias{dbGetStatement-methods}
\alias{dbGetRowCount-methods}
\alias{dbGetRowsAffected-methods}
\alias{dbColumnInfo-methods}
\alias{dbHasCompleted-methods}
\alias{dbGetInfo,PostgreSQLObject-method}
\alias{dbGetInfo,PostgreSQLDriver-method}        % BUG: this is not needed
\alias{dbGetInfo,PostgreSQLConnection-method}        % BUG: this is not needed
\alias{dbGetInfo,PostgreSQLResult-method}        % BUG: this is not needed
\alias{dbGetStatement,PostgreSQLResult-method}
\alias{dbGetRowCount,PostgreSQLResult-method}
\alias{dbGetRowsAffected,PostgreSQLResult-method}
\alias{dbColumnInfo,PostgreSQLResult-method}
\alias{dbHasCompleted,PostgreSQLResult-method}
\title{
  Database interface meta-data
}
\description{
   These methods are straight-forward implementations of the corresponding
   generic functions.
}
\section{Methods}{\describe{
\item{dbObj}{
  any object that implements some functionality in the R/S-Plus
  interface to databases (a driver, a connection or a result set).
  }
\item{res}{ an \code{PostgreSQLResult}.}
\item{\dots}{currently not being used.}
}
}
\references{
  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
  \code{\link{PostgreSQL}},
  \code{\link[DBI]{dbDriver}},
  \code{\link[DBI]{dbConnect}},
  \code{\link[DBI]{dbSendQuery}},
  \code{\link[DBI]{dbGetQuery}},
  \code{\link[DBI]{fetch}},
  \code{\link[DBI]{dbCommit}},
  \code{\link[DBI]{dbGetInfo}},
  \code{\link[DBI]{dbListTables}},
  \code{\link[DBI]{dbReadTable}}.
}
\examples{\dontrun{
drv <- dbDriver("PostgreSQL")
con <- dbConnect(drv, user= "user", password="password", dbname="sample")

dbListTables(con)

rs <- dbSendQuery(con, query.sql)
dbGetStatement(rs)
dbHasCompleted(rs)

info <- dbGetInfo(rs)
names(dbGetInfo(drv))

# DBIConnection info
names(dbGetInfo(con))

# DBIResult info
names(dbGetInfo(rs)) 
}
}
\note{nullOk in \code{dbColumnInfo} was changed. Now it may be TRUE, FALSE, or NA; the column may be totally deleted in future releases;}
\keyword{methods}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: make.db.names-methods.Rd,v 0.1 2008/07/22 03:16:22 psk Exp $

\name{make.db.names-methods}
\docType{methods}
\alias{SQLKeywords-methods}
\alias{isSQLKeyword-methods}
\alias{make.db.names,PostgreSQLObject,character-method}
\alias{SQLKeywords,PostgreSQLObject-method}
\alias{SQLKeywords,missing-method}
\alias{isSQLKeyword,PostgreSQLObject,character-method}
\title{
  Make R/S-Plus identifiers into quoted PostgreSQL identifiers
}
\description{
   Calls postgresqlquoteId to make valid quoted identifiers.
   This has calling convention same as the make.db.names for compatibility.
}
\section{Methods}{\describe{
\item{dbObj}{
  any PostgreSQL object (e.g., \code{PostgreSQLDriver}). Just ignored.
  }
\item{snames}{
  a character vector of R/S-Plus identifiers (symbols) from which
  we need to make SQL identifiers.
  }
\item{name}{
  a character vector of SQL identifiers we want to check against
  keywords from the DBMS. Ignored.
}
\item{unique}{
  logical describing whether the resulting set of SQL names should
  be unique.  Its default is \code{TRUE}.  Following the SQL 92
  standard, uniqueness of SQL identifiers is determined regardless
  of whether letters are upper or lower case. Ignored.
  }
\item{allow.keywords }{
  logical describing whether SQL keywords should be allowed in the
  resulting set of SQL names.  Its default is \code{TRUE}. Ignored.
  }
\item{keywords}{
  a character vector with SQL keywords, by default it is
  \code{.PostgreSQLKeywords} define in \code{RPostgreSQL}. This may
  be overriden by users. Ignored.
  }
\item{case}{
  a character string specifying whether to make the comparison
  as lower case, upper case, or any of the two.
  it defaults to \code{any}. Ignored.
  }
\item{\dots}{currently not used.}
}
}
\references{
  The set of SQL keywords is stored in the character vector
  \code{.SQL92Keywords} and reflects the SQL ANSI/ISO standard as
  documented in "X/Open SQL and RDA", 1994, ISBN 1-872630-68-8.
  Users can easily override or update this vector.

  PostgreSQL does add some keywords to the SQL 92 standard, they are
  listed in the \code{.PostgreSQLKeywords} object.

  See the Database Interface definition document
  \code{DBI.pdf} in the base directory of this package
  or \url{https://cran.r-project.org/package=DBI}.
}
\seealso{
  \code{\link{PostgreSQL}},
  \code{\link[DBI]{dbReadTable}},
  \code{\link[DBI]{dbWriteTable}},
  \code{\link[DBI]{dbExistsTable}},
  \code{\link[DBI]{dbRemoveTable}},
  \code{\link[DBI]{dbListTables}}.
}
\examples{\dontrun{
# This example shows how we could export a bunch of data.frames
# into tables on a remote database.

}
}
\keyword{methods}
\keyword{interface}
\keyword{database}
% vim: syntax=tex

% $Id: S4R.Rd,v 0.1 2008/07/22 20:56:01 psk Exp $
\name{S4R}
\alias{ErrorClass}
\alias{usingR}

\title{R compatibility with S version 4/Splus5+ support functions}
\description{
  These objects ease the task of porting functions into R
  from S Version 4 and S-Plus 5.0 and later.  See the documentation
  there. May be obsolete in the future.
}
\usage{
usingR(major, minor)
}
\examples{\dontrun{
rc <- try(fetch(res, n = -1))
if(inherit(rc, ErrorClass))
   stop("could not fetch the data")
}
}
\keyword{internal}
% vim:syntax=tex

% $Id: dbObjectId-class.Rd,v 0.1 2008/08/10 18:04:01 psk Exp $
\name{dbObjectId-class}
\docType{class}
\alias{dbObjectId-class}
\title{Class dbObjectId}
\description{  
   A helper (mixin) class to provide external references in
   an R/S-Plus portable way.
}
\section{Objects from the Class}{A virtual Class: No objects may be created from it.}
\section{Slots}{
  \describe{
    \item{\code{Id}:}{Object of class \code{"integer"} 
    this is an integer vector holding an opaque reference into a C struct
    (may or may not be a C pointer, may or may not have length one).
    }
  }
}

\section{Methods}{
  \describe{
    \item{\link{coerce}}{\code{signature(from = "dbObjectId", to = "integer")}: ... }
    \item{\link{coerce}}{\code{signature(from = "dbObjectId", to = "numeric")}: ... }
    \item{\link{coerce}}{\code{signature(from = "dbObjectId", to = "character")}: ... }
    \item{\link{format}}{\code{signature(x = "dbObjectId")}: ... }
    \item{\link{print}}{\code{signature(x = "dbObjectId")}: ... }
    \item{\link{show}}{\code{signature(object = "dbObjectId")}: ... }
  }
}

\note{A cleaner mechanism would use external references, but
historically this class has existed mainly for R/S-Plus portability.}

\examples{\dontrun{
  pg <- dbDriver("PostgreSQL")
  con <- dbConnect(pg, "user", "password")
  is(pg, "dbObjectId")   ## True
  is(con, "dbObjectId")  ## True
  isPostgresqlIdCurrent(con)       ## True
  q("yes")
  \$ R 
  isPostgresqlIdCurrent(con)       ## False
}
}
\keyword{classes}
\keyword{interface}
\keyword{database}
% vim: syntax=tex
